# -*- coding: utf-8 -*-
from enum import Enum, IntEnum, unique


@unique
class UserFormInfoFlag(Enum):
    """微信卡券会员卡格式化的选项类型"""

    MOBILE = "USER_FORM_INFO_FLAG_MOBILE"  # 手机号
    SEX = "USER_FORM_INFO_FLAG_SEX"  # 性别
    NAME = "USER_FORM_INFO_FLAG_NAME"  # 姓名
    BIRTHDAY = "USER_FORM_INFO_FLAG_BIRTHDAY"  # 生日
    IDCARD = "USER_FORM_INFO_FLAG_IDCARD"  # 身份证
    EMAIL = "USER_FORM_INFO_FLAG_EMAIL"  # 邮箱
    LOCATION = "USER_FORM_INFO_FLAG_LOCATION"  # 详细地址
    EDUCATION_BACKGRO = "USER_FORM_INFO_FLAG_EDUCATION_BACKGRO"  # 教育背景
    INDUSTRY = "USER_FORM_INFO_FLAG_INDUSTRY"  # 行业
    INCOME = "USER_FORM_INFO_FLAG_INCOME"  # 收入
    HABIT = "USER_FORM_INFO_FLAG_HABIT"  # 兴趣爱好


@unique
class ReimburseStatus(Enum):
    """发票报销状态"""

    INIT = "INVOICE_REIMBURSE_INIT"  # 初始状态，未锁定，可提交报销
    LOCK = "INVOICE_REIMBURSE_LOCK"  # 已锁定，无法重复提交报销
    CLOSURE = "INVOICE_REIMBURSE_CLOSURE"  # 已核销，从用户卡包中移除


@unique
class WeChatErrorCode(IntEnum):
    """
    微信接口返回码，全局返回码请参考 https://developers.weixin.qq.com/doc/offiaccount/Getting_Started/Global_Return_Code.html
    """

    # 系统错误
    SYSTEM_ERROR = -1000

    # 系统繁忙
    # 此时请开发者稍候再试
    SYSTEM_BUSY = -1

    # 请求成功
    SUCCESS = 0

    # AppSecret 错误，或是 Access Token 无效
    # 请开发者认真比对AppSecret的正确性，或查看是否正在为恰当的公众号调用接口
    INVALID_CREDENTIAL = 40001

    # 错误的凭证类型
    INVALID_CREDENTIAL_TYPE = 40002

    # 错误的 OpenID
    # 请开发者确认 OpenID 是否已关注公众号，或是否是其他公众号的 OpenID
    INVALID_OPENID = 40003

    # 不支持的媒体文件类型
    INVALID_MEDIA_TYPE = 40004

    # 不支持的文件类型
    INVALID_FILE_TYPE = 40005

    # 不支持的文件大小
    INVALID_FILE_SIZE = 40006

    # 错误的 MediaID
    INVALID_MEDIA_ID = 40007

    # 错误的消息类型
    INVALID_MESSAGE_TYPE = 40008

    # 不支持的图片大小
    # 图片格式不对有时也会报这个错
    INVALID_IMAGE_SIZE = 40009

    # 不支持的语音文件大小
    INVALID_VOICE_SIZE = 40010

    # 不支持的视频文件大小
    INVALID_VIDEO_SIZE = 40011

    # 不支持的缩略图大小
    INVALID_THUMB_SIZE = 40012

    # 错误的 AppID
    # 目前 AppID 格式都是 /^wx\d{16}$/
    INVALID_APP_ID = 40013

    # 不合法的 Access Token
    # 请开发者认真比对 Access Token 的有效性（如是否过期），或查看是否正在为恰当的公众号调用接口
    INVALID_ACCESS_TOKEN = 40014

    # 错误的按钮类型
    INVALID_BUTTON_TYPE = 40015

    # 不支持的主菜单按钮个数
    # 微信自定义菜单按钮个数应该在 1~3 个之间
    INVALID_BUTTON_SIZE = 40016

    # 不支持的子菜单按钮个数
    # 微信自定义子菜单按钮个数应该在 1~5 个之间
    INVALID_SUB_BUTTON_SIZE = 40017

    # 不支持的按钮名字长度
    INVALID_BUTTON_NAME_SIZE = 40018

    # 不支持的按钮 key 长度
    INVALID_BUTTON_KEY_SIZE = 40019

    # 不支持的按钮 url 长度
    INVALID_BUTTON_URL_SIZE = 40020

    # 不合法的菜单版本号
    INVALID_MENU_VERSION = 40021

    # 不合法的子菜单级数
    INVALID_SUB_BUTTON_LEVEL = 40022

    # 不合法的子菜单按钮个数
    INVALID_SUB_BUTTON_COUNT = 40023

    # 不合法的子菜单按钮类型
    INVALID_SUB_BUTTON_TYPE = 40024

    # 不合法的子菜单按钮名字长度
    INVALID_SUB_BUTTON_NAME_SIZE = 40025

    # 不合法的子菜单按钮 key 长度
    INVALID_SUB_BUTTON_KEY_SIZE = 40026

    # 不合法的子菜单按钮 url 长度
    INVALID_SUB_BUTTON_URL_SIZE = 40027

    # 不合法的自定义菜单使用用户
    INVALID_MENU_USER = 40028

    # 错误的 OAuth Code
    INVALID_OAUTH_CODE = 40029

    # 错误的 Refresh Token
    INVALID_REFRESH_TOKEN = 40030

    # 错误的 OpenID 列表
    INVALID_OPENID_LIST = 40031

    # 错误的 OpenID 列表长度
    # 列表内最多10000个 OpenID
    INVALID_OPENID_LIST_SIZE = 40032

    # 不支持的请求字符
    # 不能包含 \uxxxx 格式的字符
    INVALID_REQUEST_CHARSET = 40033

    # 不合法的参数
    INVALID_PARAMETER = 40035

    # 错误的模板消息 ID
    # Template ID 失效了，请重新刷新一次 Template ID
    INVALID_TEMPLATE = 40037

    # 不合法的请求格式
    INVALID_REQUEST_FORMAT = 40038

    # 不合法的 url 长度
    INVALID_URL_SIZE = 40039

    # 无效的 url
    INVALID_URL_DOMAIN = 40048

    # 不合法的分组 ID
    INVALID_GROUP_ID = 40050

    # 不合法的分组名字
    # 40117 也是这个错误
    INVALID_GROUP_NAME = 40051

    # 不支持的操作
    # 可能是该公众号已经申请完了十万个二维码
    INVALID_ACTION_INFO = 40053

    # 自定义菜单的按钮里，网址有误
    INVALID_BUTTON_DOMAIN = 40054

    # 自定义子菜单的按钮里，网址有误
    INVALID_SUB_BUTTON_DOMAIN = 40055

    # 删除单篇图文时，指定的 article_idx 不合法
    INVALID_DELETE_ARTICLE_ID = 40060

    # 错误的行业号
    # 有一些模板消息只会在特定的行业下申请
    INVALID_INDUSTRY_ID = 40102

    # 不支持的 MediaID 长度
    INVALID_MEDIA_ID_SIZE = 40118

    # button 类型错误
    INVALID_USE_BUTTON_TYPE = 40119

    # 子 button 类型错误
    INVALID_USE_SUB_BUTTON_TYPE = 40120

    # 不支持的 MediaID 类型
    INVALID_MEDIA_ID_TYPE = 40121

    # 无效的 AppSecret
    INVALID_APP_SECRET = 40125

    # 微信号不合法
    INVALID_WECHAT_ID = 40132

    # 不支持的图片格式
    INVALID_IMAGE_FORMAT = 40137

    # 请勿添加其他公众号的主页链接
    CONTAIN_OTHER_HOME_PAGE_URL = 40155

    # OAuth Code 已使用
    CODE_BEEN_USED = 40163

    # 缺少 Access Token 参数
    MISSING_ACCESS_TOKEN = 41001

    # 缺少 AppID 参数
    MISSING_APP_ID = 41002

    # 缺少 Refresh Token 参数
    MISSING_REFRESH_TOKEN = 41003

    # 缺少 AppSecret 参数
    MISSING_APP_SECRET = 41004

    # 缺少多媒体文件数据
    MISSING_MEDIA_DATA = 41005

    # 缺少 MediaID 参数
    MISSING_MEDIA_ID = 41006

    # 缺少子菜单数据
    MISSING_SUB_BUTTONS = 41007

    # 缺少 OAuth Code
    MISSING_OAUTH_CODE = 41008

    # 缺少 OpenID
    MISSING_OPENID = 41009

    # page 路径不正确，需要保证在现网版本小程序中存在，与 app.json 保持一致
    INVALID_PAGE = 41030

    # Access Token 已失效
    # 请检查 Access Token 的有效期，重新刷新 Access Token
    EXPIRED_ACCESS_TOKEN = 42001

    # Refresh Token 已失效
    EXPIRED_REFRESH_TOKEN = 42002

    # OAuth Code 已失效
    EXPIRED_OAUTH_CODE = 42003

    # 授权已失效
    # 用户修改微信密码，Access Token, Refresh Token 均已失效，需要重新授权
    EXPIRED_AUTHORIZATION = 42007

    # 需要 Get 请求
    REQUIRE_GET = 43001

    # 需要 Post 请求
    REQUIRE_POST = 43002

    # 需要 Https 请求
    REQUIRE_HTTPS = 43003

    # 用户没有关注公众号
    REQUIRE_SUBSCRIBE = 43004

    # 需要好友关系
    REQUIRE_FRIEND = 43005

    # 用户被拉黑
    # 需要公众号把该用户从黑名单里移除
    REQUIRE_UNBLOCK_USER = 43019

    # 超过了更换行业的限制
    # 一个月最多换一次
    OUT_OF_CHANGE_INDUSTRY_LIMIT = 43100

    # 用户拒绝接受消息，如果用户之前曾经订阅过，则表示用户取消了订阅关系
    USER_REFUSE_TO_ACCEPT_THE_MESSAGE = 43101

    # 多媒体文件为空
    EMPTY_MEDIA_DATA = 44001

    # POST 的数据包为空
    EMPTY_POST_DATA = 44002

    # 图文消息内容为空
    EMPTY_NEWS_DATA = 44003

    # 文本消息内容为空
    EMPTY_CONTENT = 44004

    # 多媒体文件大小超过限制
    # 最大允许 1MB
    OUT_OF_MEDIA_SIZE_LIMIT = 45001

    # 消息内容超过限制
    OUT_OF_CONTENT_SIZE_LIMIT = 45002

    # 标题长度超过限制
    # 最长允许 64 字符长度
    OUT_OF_TITLE_SIZE_LIMIT = 45003

    # 描述字段超过限制
    OUT_OF_DESCRIPTION_SIZE_LIMIT = 45004

    # 链接字段超过限制
    OUT_OF_URL_SIZE_LIMIT = 45005

    # 图片链接字段超过限制
    OUT_OF_PIC_URL_SIZE_LIMIT = 45006

    # 语音播放时间超过限制
    # 最长允许 60 秒
    OUT_OF_VOICE_TIME_LIMIT = 45007

    # 图文消息数量超过限制
    # 最多 10 条图文消息
    OUT_OF_ARTICLE_SIZE_LIMIT = 45008

    # 接口调用频率超过限制
    OUT_OF_API_FREQ_LIMIT = 45009

    # 创建菜单个数超过限制
    OUT_OF_MENU_SIZE_LIMIT = 45010

    # API 调用太频繁，请稍候再试
    API_MINUTE_QUOTA_REACH_LIMIT = 45011

    # 回复时间超过限制
    # 接受推送后，5 秒内未被动响应。或者是用户与公众号 48 小时无互动后，调用客服接口主动推送消息。
    OUT_OF_RESPONSE_TIME_LIMIT = 45015

    # 系统分组，不允许修改
    SYSTEM_GROUP_CANNOT_CHANGE = 45016

    # 分组名字过长
    OUT_OF_GROUP_NAME_SIZE_LIMIT = 45017

    # 分组数量超过上限
    OUT_OF_GROUP_SIZE_LIMIT = 45018

    # 模板消息数量超过限制
    OUT_OF_TEMPLATE_SIZE_LIMIT = 45026

    # 模板消息与行业信息冲突
    TEMPLATE_CONFLICT_WITH_INDUSTRY = 45027

    # 客服接口下行条数超过上限
    OUT_OF_RESPONSE_COUNT_LIMIT = 45047

    # 创建菜单包含未关联的小程序
    NO_PERMISSION_TO_USE_WEAPP_IN_MENU = 45064

    # 相同 clientmsgid 已存在群发记录，返回数据中带有已存在的群发任务的 msgid
    CLIENTMSGID_EXIST = 45065

    #  相同 clientmsgid 重试速度过快，请间隔1分钟重试
    OUT_OF_CLIENTMSGID_API_FREQ_LIMIT = 45066

    # clientmsgid 长度超过限制
    CLIENTMSGID_SIZE_OUT_OF_LIMIT = 45067

    # 不支持的图文消息内容
    # 请确认 content 里没有超链接标签
    INVALID_CONTENT = 45166

    # 不存在媒体数据
    MEDIA_DATA_NO_EXIST = 46001

    # 不存在的菜单版本
    MENU_VERSION_NOT_EXIST = 46002

    # 不存在的菜单数据
    MENU_NO_EXIST = 46003

    # 不存在的用户
    USER_NOT_EXIST = 46004

    # 解析 JSON/XML 内容错误
    DATA_FORMAT_ERROR = 47001

    # 模板参数不准确，可能为空或者不满足规则，errmsg 会提示具体是哪个字段出错
    INVALID_TEMPLATE_ARGUMENT = 47003

    # API 功能未授权
    # 请确认公众号已获得该接口，可以在公众平台官网-开发者中心页中查看接口权限
    UNAUTHORIZED_API = 48001

    # 用户拒收公众号消息
    # (在公众号选项中，关闭了“接收消息”)
    USER_BLOCK_MESSAGE = 48002

    # 公众号管理员没有同意微信群发协议
    # 请登录公众号后台点一下同意
    USER_NOT_AGREE_PROTOCOL = 48003

    # API 接口被封禁
    # 请登录公众号后台查看详情
    API_BANNED = 48004

    # API 禁止删除被自动回复和自定义菜单引用的素材
    API_DELETE_PROHIBITED = 48005

    # API 清零次数失败，因为清零次数达到上限
    OUT_OF_RESET_LIMIT = 48006

    # 没有该类型消息的发送权限
    NO_PERMISSION_FOR_THIS_MSGTYPE = 48008

    # 用户未授权该 API
    USER_UNAUTHORIZED = 50001

    # 用户受限，可能是违规后接口被封禁
    USER_LIMITED = 50002

    # 用户未关注的公众号
    UNSUBSCRIBE_OFFICIAL_ACCOUNT = 50005

    # 发布功能被封禁
    PUBLISH_LIMITED = 53500

    # 频繁请求发布
    OUT_OF_PUBLISH_LIMIT = 53501

    # Publish ID 无效
    INVALID_PUBLISH_ID = 53502

    # Article ID 无效
    INVALID_ARTICLE_ID = 53600

    # 公众号未授权给开放平台
    UNAUTHORIZED_COMPONENT = 61003

    # 公众号未授权该 API 给开放平台
    UNAUTHORIZED_COMPONENT_API = 61007

    # 错误的开放平台 Refresh Token
    INVALID_COMPONENT_REFRESH_TOKEN = 61023

    # 参数错误
    ERROR_PARAMETER = 61451

    # 无效客服账号
    INVALID_KF_ACCOUNT = 61452

    # 客服帐号已存在
    KF_ACCOUNT_EXISTED = 61453

    # 客服帐号名长度超过限制 ( 仅允许 10 个英文字符，不包括 @ 及 @ 后的公众号的微信号 )
    INVALID_KF_ACCOUNT_LENGTH = 61454

    # 客服帐号名包含非法字符 ( 仅允许英文 + 数字 )
    ILLEGAL_CHARTER_IN_KF_ACCOUNT = 61455

    # 客服帐号个数超过限制 (10 个客服账号 )
    KF_ACCOUNT_EXCEEDED = 61456

    # 无效头像文件类型
    INVALID_AVATAR_FILE_TYPE = 61457

    # 日期格式错误
    DATE_FORMAT_ERROR = 61500

    # 部分参数为空
    MISSING_PARAMETER = 63001

    # 无效的 JS SDK 签名
    INVALID_JS_SDK_SIGNATURE = 63002

    # 不存在此 menuid 对应的个性化菜单
    INVALID_MENU_ID = 65301

    # 没有默认菜单，不能创建个性化菜单
    THERE_IS_NO_SELFMENU = 65303

    # MatchRule 信息为空
    MATCH_RULE_EMPTY = 65304

    # 个性化菜单数量受限
    MENU_COUNT_LIMIT = 65305

    # 不支持个性化菜单的帐号
    INVALID_ACCOUNT_FOR_MENU = 65306

    # 个性化菜单信息为空
    EMPTY_MENU = 65307

    # 包含没有响应类型的 button
    BUTTON_MISSING_RESPONSE = 65308

    # 个性化菜单开关处于关闭状态
    DISABLE_MENU = 65309

    # 填写了省份或城市信息，国家信息不能为空
    MISSING_COUNTRY = 65310

    # 填写了城市信息，省份信息不能为空
    MISSING_PROVINCE = 65311

    # 不合法的国家信息
    INVALID_COUNTRY = 65312

    # 不合法的省份信息
    INVALID_PROVINCE = 65313

    # 不合法的城市信息
    INVALID_CITY_INFO = 65314

    # 该公众号的菜单设置了过多的域名外跳（最多跳转到 3 个域名的链接）
    DOMAIN_COUNT_REACH_LIMIT = 65316

    # 不合法的 URL
    INVALID_URL = 65317

    # 无效的签名
    INVALID_SIGNATURE = 87009

    # 内容可能潜在风险
    RISKY_CONTENT = 87014

    # POST 数据参数不合法
    INVALID_POST_DATA = 9001001

    # 远端服务不可用
    REMOTE_SERVICE_UNAVAILABLE = 9001002

    # Ticket 不合法
    INVALID_TICKET = 9001003

    # 获取摇周边用户信息失败
    GET_USER_FAILED = 9001004

    # 获取商户信息失败
    GET_MERCHANT_FAILED = 9001005

    # 获取 OpenID 失败
    GET_OPENID_FAILED = 9001006

    # 上传文件缺失
    UPLOAD_FILE_MISSING = 9001007

    # 上传素材的文件类型不合法
    UPLOAD_FILE_TYPE_ERROR = 9001008

    # 上传素材的文件尺寸不合法
    UPLOAD_FILE_SIZE_ERROR = 9001009

    # 上传失败
    UPLOAD_FAILED = 9001010

    # 帐号不合法
    INVALID_ACCOUNT = 9001020

    # 已有设备激活率低于 50% ，不能新增设备
    ACTIVE_DEVICE_LESS_THAN_HALF = 9001021

    # 设备申请数不合法，必须为大于 0 的数字
    INVALID_DEVICE_COUNT = 9001022

    # 已存在审核中的设备 ID 申请
    DEVICE_ID_EXISTED = 9001023

    # 一次查询设备 ID 数量不能超过 50
    OUT_OF_DEVICE_QUERY_LIMIT = 9001024

    # 设备 ID 不合法
    INVALID_DEVICE_ID = 9001025

    # 页面 ID 不合法
    INVALID_PAGE_ID = 9001026

    # 页面参数不合法
    INVALID_PAGE_PARAMETER = 9001027

    # 一次删除页面 ID 数量不能超过 10
    OUT_OF_PAGE_DELETE_LIMIT = 9001028

    # 页面已应用在设备中，请先解除应用关系再删除
    PAGE_APPLIED_IN_DEVICE = 9001029

    # 一次查询页面 ID 数量不能超过 50
    OUT_OF_PAGE_QUERY_LIMIT = 9001030

    # 时间区间不合法
    INVALID_TIME_RANGE = 9001031

    # 保存设备与页面的绑定关系参数错误
    BIND_DEVICE_PAGE_PARAMETER_ERROR = 9001032

    # 门店 ID 不合法
    INVALID_LOCATION_ID = 9001033

    # 设备备注信息过长
    OUT_OF_DEVICE_DESC_LENGTH_LIMIT = 9001034

    # 设备申请参数不合法
    INVALID_DEVICE_PARAMETER = 9001035

    # 查询起始值 begin 不合法
    INVALID_BEGIN = 9001036



# -*- coding: utf-8 -*-

import random
from datetime import datetime

from wechatpy.pay.utils import get_external_ip
from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatMicroPay(BaseWeChatPayAPI):
    def create(
        self,
        body,
        total_fee,
        auth_code,
        client_ip=None,
        out_trade_no=None,
        detail=None,
        attach=None,
        fee_type="CNY",
        goods_tag=None,
        device_info=None,
        limit_pay=None,
        receipt="Y",
    ):
        """
        刷卡支付接口
        :param device_info: 可选，终端设备号(商户自定义，如门店编号)
        :param body: 商品描述
        :param detail: 可选，商品详情
        :param attach: 可选，附加数据，在查询API和支付通知中原样返回，该字段主要用于商户携带订单的自定义数据
        :param client_ip: 可选，APP和网页支付提交用户端ip，Native支付填调用微信支付API的机器IP
        :param out_trade_no: 可选，商户订单号，默认自动生成
        :param total_fee: 总金额，单位分
        :param fee_type: 可选，符合ISO 4217标准的三位字母代码，默认人民币：CNY
        :param goods_tag: 可选，商品标记，代金券或立减优惠功能的参数
        :param limit_pay: 可选，指定支付方式，no_credit--指定不能使用信用卡支付
        :param auth_code: 授权码，扫码支付授权码，设备读取用户微信中的条码或者二维码信息
        :param receipt: Y，传入Y时，支付成功消息和支付详情页将出现开票入口。需要在微信支付商户平台或微信公众平台开通电子发票功能，传此字段才可生效
        :return: 返回的结果数据
        """
        now = datetime.now()
        if not out_trade_no:
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "appid": self.appid,
            "device_info": device_info,
            "body": body,
            "detail": detail,
            "attach": attach,
            "out_trade_no": out_trade_no,
            "total_fee": total_fee,
            "fee_type": fee_type,
            "spbill_create_ip": client_ip or get_external_ip(),
            "goods_tag": goods_tag,
            "limit_pay": limit_pay,
            "auth_code": auth_code,
            "receipt": receipt,
        }
        return self._post("pay/micropay", data=data)

    def query(self, transaction_id=None, out_trade_no=None):
        """
        查询订单

        :param transaction_id: 微信的订单号，优先使用
        :param out_trade_no: 商户系统内部的订单号，当没提供transaction_id时需要传这个。
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "transaction_id": transaction_id,
            "out_trade_no": out_trade_no,
        }
        return self._post("pay/orderquery", data=data)



# -*- coding: utf-8 -*-
import operator
from typing import Dict, List, Tuple

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatTemplate(BaseWeChatAPI):
    """包含了模板消息(旧)以及订阅通知(新)的接口，以

    模板消息: https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html
    订阅通知: https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/intro.html

    旧的模板消息，接口不变
    新的订阅通知模板，类似官方对订阅消息的取名 SubscribeMsg, 我们以 subscribe_message_template 命名予以区分
    """

    API_BASE_URL = "https://api.weixin.qq.com/"

    def set_industry(self, industry_id1, industry_id2):
        """
        【模板消息】设置所属行业
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#0

        :param industry_id1: 公众号模板消息所属行业编号
        :param industry_id2: 公众号模板消息所属行业编号
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "cgi-bin/template/api_set_industry",
            data={"industry_id1": industry_id1, "industry_id2": industry_id2},
        )

    def get_industry(self):
        """
        【模板消息】获取设置的行业信息
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#1

        :return: 返回的 JSON 数据包
        """
        return self._get("cgi-bin/template/get_industry")

    def get(self, template_id_short):
        """
        【模板消息】获得模板ID
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#2

        :param template_id_short: 模板库中模板的编号，有“TM**”和“OPENTMTM**”等形式
        :return: 模板 ID
        """
        res = self._post(
            "cgi-bin/template/api_add_template",
            data={"template_id_short": template_id_short},
            result_processor=operator.itemgetter("template_id"),
        )
        return res

    add = get

    def get_all_private_template(self):
        """
        【模板消息】获取模板列表
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#3

        :return: 返回的 JSON 数据包
        """
        return self._get("cgi-bin/template/get_all_private_template")

    def del_private_template(self, template_id):
        """
        【模板消息】删除模板
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Template_Message_Interface.html#4

        :param template_id: 公众帐号下模板消息ID
        :return: 返回的 JSON 数据包
        """
        return self._post("cgi-bin/template/del_private_template", data={"template_id": template_id})

    def add_subscribe_message_template(self, tid: str, keywords: List[int], description: str) -> str:
        """
        【订阅通知】选用模板 (使用 tid 换取 template_id)
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF

        返回：添加至帐号下的模板 template_id，发送订阅通知时所需
        """
        return self._post(
            "wxaapi/newtmpl/addtemplate",
            data={
                "tid": tid,
                "kidList": keywords,
                "sceneDesc": description,
            },
            result_processor=operator.itemgetter("priTmplId"),
        )

    def del_subscribe_message_template(self, template_id: str):
        """
        【订阅通知】删除模板 (删除 template_id)
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF
        """
        return self._post("wxaapi/newtmpl/deltemplate", data={"priTmplId": template_id})

    def get_category(self) -> List[Dict]:
        """
        【订阅通知】获取公众号类目
        详情请参考:
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF

        返回数据参考:
        [
          { "id": 616, "name": "公交" }
        ]
        """
        return self._get("wxaapi/newtmpl/getcategory", result_processor=operator.itemgetter("data"))

    def get_subscribe_message_template_keywords(self, tid: str) -> Tuple[int, List[Dict]]:
        """
        【订阅通知】获取模板中的关键词
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF

        返回数据参考:
        (
          10,  # 公共模板列表总数
          [    # 关键词列表
            { "kid": 1, "name": "物品名称", "example": "名称", "rule": "thing" }
          ]
        )
        """
        return self._get(
            "wxaapi/newtmpl/getpubtemplatekeywords",
            params={"tid": tid},
            result_processor=operator.itemgetter("count", "data"),
        )

    def get_subscribe_message_template_titles(self, start: int = 0, limit: int = 30) -> Tuple[int, List[Dict]]:
        """
        【订阅通知】获取所属类目的公共模板
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF

        返回数据参考:
        (
          10,  # 公共模板列表总数
          [    # 模板标题列表
            { "tid": 99, "title": "付款成功通知", "type": 2, "categoryId": "616" }
          ]
        )
        """
        return self._get(
            "wxaapi/newtmpl/getpubtemplatetitles",
            params={"start": start, "limit": limit},
            result_processor=operator.itemgetter("count", "data"),
        )

    def get_subscribe_message_templates(self) -> List[Dict]:
        """
        【订阅通知】获取私有模板列表
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Subscription_Messages/api.html#addTemplate%E9%80%89%E7%94%A8%E6%A8%A1%E6%9D%BF

        返回数据参考:
        [
          {
            "priTmplId": "9Aw5ZV1j9xdWTFEkqCpZ7mIBbSC34khK55OtzUPl0rU",
            "title": "报名结果通知",
            "content": "会议时间:{{date2.DATA}}\n会议地点:{{thing1.DATA}}\n",
            "example": "会议时间:2016年8月8日\n会议地点:TIT会议室\n",
            "type": 2
          }
        ]
        """
        return self._get("wxaapi/newtmpl/gettemplate", result_processor=operator.itemgetter("data"))

    # send 接口参见 wechatpy.client.api.message.WeChatMessage.send_subscribe_message



# -*- coding: utf-8 -*-

from typing import Optional, List, Dict, Any

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatTag(BaseWeChatAPI):
    @staticmethod
    def _validate_tag_id(tag_id):
        if tag_id < 0:
            raise ValueError("tag id cannot be a negative integer")

    @staticmethod
    def _validate_tag_name(tag_name):
        if len(tag_name) > 32:
            raise ValueError("the length of the tag name cannot be more than 32 characters")

    def create(self, name: str, tag_id: Optional[int] = None) -> dict:
        """创建标签

        参考：https://developer.work.weixin.qq.com/document/path/90210

        **权限说明**： 创建的标签属于该应用，只有该应用才可以增删成员。

        **注意**： 标签总数不能超过3000个。

        返回结果示例: ::

            {
               "errcode": 0,
               "errmsg": "created"
               "tagid": 12
            }

        返回结果参数说明：

        +---------+------------------------+
        | 参数    | 说明                   |
        +=========+========================+
        | errcode | 返回码                 |
        +---------+------------------------+
        | errmsg  | 对返回码的文本描述内容 |
        +---------+------------------------+
        | tagid   | 标签id                 |
        +---------+------------------------+

        :param name: 标签名称，长度限制为32个字以内（汉字或英文字母），标签名不可与
            其他标签重名。
        :param tag_id: 标签id，非负整型，指定此参数时新增的标签会生成对应的标签id，不指
            定时则以目前最大的id自增。
        :return: 创建结果
        """
        if tag_id is not None:
            self._validate_tag_id(tag_id)
        self._validate_tag_name(name)

        data: Dict[str, Any] = {"tagname": name}
        if tag_id:
            data["tagid"] = tag_id
        return self._post("tag/create", data=data)

    def update(self, tag_id: int, name: str) -> dict:
        """更新标签名字

        参考：https://developer.work.weixin.qq.com/document/path/90211

        **权限说明**：调用的应用必须是指定标签的创建者。

        返回结果示例： ::

            {
               "errcode": 0,
               "errmsg": "updated"
            }

        结果参数说明：

        +---------+------------------------+
        | 参数    | 说明                   |
        +=========+========================+
        | errcode | 返回码                 |
        +---------+------------------------+
        | errmsg  | 对返回码的文本描述内容 |
        +---------+------------------------+

        :param tag_id: 标签ID
        :param name: 标签名称，长度限制为32个字（汉字或英文字母），标签不可与其他标签重名。
        :return: 更新结果
        """
        self._validate_tag_id(tag_id)
        self._validate_tag_name(name)

        return self._post("tag/update", data={"tagid": tag_id, "tagname": name})

    def delete(self, tag_id: int) -> dict:
        """删除标签

        参考：https://developer.work.weixin.qq.com/document/path/90212

        **权限说明**：调用的应用必须是指定标签的创建者。

        返回结果示例： ::

            {
               "errcode": 0,
               "errmsg": "deleted"
            }

        结果参数说明：

        +---------+------------------------+
        | 参数    | 说明                   |
        +=========+========================+
        | errcode | 返回码                 |
        +---------+------------------------+
        | errmsg  | 对返回码的文本描述内容 |
        +---------+------------------------+

        :param tag_id: 标签ID，非负整型
        :return: 删除结果
        """
        self._validate_tag_id(tag_id)
        return self._get("tag/delete", params={"tagid": tag_id})

    def get_users(self, tag_id: int) -> dict:
        """获取标签成员

        参考：https://developer.work.weixin.qq.com/document/path/90213

        **权限说明**：
        无限制，但返回列表仅包含应用可见范围的成员；第三方可获取自己创建的标签及应用可见
        范围内的标签详情。

        返回结果示例： ::

            {
               "errcode": 0,
               "errmsg": "ok",
               "tagname": "乒乓球协会",
               "userlist": [
                     {
                         "userid": "zhangsan",
                         "name": "李四"
                     }
                 ],
               "partylist": [2]
            }

        结果参数说明：

        +-----------+-------------------------------------------------------------------------------+
        | 参数      | 说明                                                                          |
        +===========+===============================================================================+
        | errcode   | 返回码                                                                        |
        +-----------+-------------------------------------------------------------------------------+
        | errmsg    | 对返回码的文本描述内容                                                        |
        +-----------+-------------------------------------------------------------------------------+
        | tagname   | 标签名                                                                        |
        +-----------+-------------------------------------------------------------------------------+
        | userlist  | 标签中包含的成员列表                                                          |
        +-----------+-------------------------------------------------------------------------------+
        | userid    | 成员帐号                                                                      |
        +-----------+-------------------------------------------------------------------------------+
        | name      | 成员名称，此字段从2019年12月30日起，对新创建第三方应用不再返回，              |
        |           | 2020年6月30日起，对所有历史第三方应用不再返回，后续第三方仅通讯录应用可获取， |
        |           | 第三方页面需要通过通讯录展示组件来展示名字                                    |
        +-----------+-------------------------------------------------------------------------------+
        | partylist | 标签中包含的部门id列表                                                        |
        +-----------+-------------------------------------------------------------------------------+

        :param tag_id: 标签ID，非负整型
        :return: 成员用户信息
        """
        self._validate_tag_id(tag_id)
        return self._get("tag/get", params={"tagid": tag_id})

    def add_users(
        self, tag_id: int, user_ids: Optional[List[str]] = None, department_ids: Optional[List[int]] = None
    ) -> dict:
        """增加标签成员

        参考：https://developer.work.weixin.qq.com/document/path/90214

        **权限说明**：
        调用的应用必须是指定标签的创建者；成员属于应用的可见范围。

        **注意**：每个标签下部门、人员总数不能超过3万个。

        返回结果示例：

        a. 正确时返回 ::

            {
               "errcode": 0,
               "errmsg": "ok"
            }

        b. 若部分userid、partylist非法，则返回 ::

            {
               "errcode": 0,
               "errmsg": "ok",
               "invalidlist"："usr1|usr2|usr",
               "invalidparty"：[2,4]
            }

        c. 当包含userid、partylist全部非法时返回 ::

            {
               "errcode": 40070,
               "errmsg": "all list invalid "
            }

        结果参数说明：

        +--------------+------------------------+
        | 参数         | 说明                   |
        +==============+========================+
        | errcode      | 返回码                 |
        +--------------+------------------------+
        | errmsg       | 对返回码的文本描述内容 |
        +--------------+------------------------+
        | invalidlist  | 非法的成员帐号列表     |
        +--------------+------------------------+
        | invalidparty | 非法的部门id列表       |
        +--------------+------------------------+

        :param tag_id: 标签ID，非负整型
        :param user_ids: 企业成员ID列表，注意：user_ids和department_ids不能同时为空，
            单次请求个数不超过1000
        :param department_ids: 企业部门ID列表，注意：user_ids和department_ids不能
            同时为空，单次请求个数不超过100
        :return: 请求结果
        """
        self._validate_tag_id(tag_id)
        if not user_ids and not department_ids:
            raise ValueError("user_ids and department_ids cannot be empty at the same time")
        if user_ids is not None and len(user_ids) > 1000:
            raise ValueError("the length of the user_ids cannot be greater than 1000")
        if department_ids is not None and len(department_ids) > 100:
            raise ValueError("the length of the department_ids cannot be greater than 100")

        data: Dict[str, Any] = {"tagid": tag_id}
        if user_ids:
            data["userlist"] = user_ids
        if department_ids:
            data["partylist"] = department_ids

        return self._post("tag/addtagusers", data=data)

    def delete_users(
        self, tag_id: int, user_ids: Optional[List[str]] = None, department_ids: Optional[List[int]] = None
    ) -> dict:
        """删除标签成员

        参考：https://developer.work.weixin.qq.com/document/path/90215

        **权限说明**：
        调用的应用必须是指定标签的创建者；成员属于应用的可见范围。

        返回结果示例：

        a. 正确时返回 ::

            {
               "errcode": 0,
               "errmsg": "deleted"
            }
        b. 若部分userid、partylist非法，则返回 ::

            {
               "errcode": 0,
               "errmsg": "deleted",
               "invalidlist"："usr1|usr2|usr",
               "invalidparty": [2,4]
            }

        c. 当包含的userid、partylist全部非法时返回 ::

            {
               "errcode": 40031,
               "errmsg": "all list invalid"
            }

        结果参数说明：

        +--------------+------------------------+
        | 参数         | 说明                   |
        +==============+========================+
        | errcode      | 返回码                 |
        +--------------+------------------------+
        | errmsg       | 对返回码的文本描述内容 |
        +--------------+------------------------+
        | invalidlist  | 非法的成员帐号列表     |
        +--------------+------------------------+
        | invalidparty | 非法的部门id列表       |
        +--------------+------------------------+

        :param tag_id: 标签ID，非负整型
        :param user_ids: 企业成员ID列表，注意：user_ids和department_ids不能同时为空，
            单次请求长度不超过1000
        :param department_ids: 企业部门ID列表，注意：user_ids和department_ids不能
            同时为空，单次请求长度不超过100
        :return: 处理结果
        """
        self._validate_tag_id(tag_id)
        if not user_ids and not department_ids:
            raise ValueError("user_ids and department_ids cannot be empty at the same time")
        if user_ids is not None and len(user_ids) > 1000:
            raise ValueError("the length of the user_ids cannot be greater than 1000")
        if department_ids is not None and len(department_ids) > 100:
            raise ValueError("the length of the department_ids cannot be greater than 100")

        data: Dict[str, Any] = {"tagid": tag_id}
        if user_ids:
            data["userlist"] = user_ids
        if department_ids:
            data["partylist"] = department_ids

        return self._post("tag/deltagusers", data=data)

    def list(self) -> List[dict]:
        """获取标签列表

        参考：https://developer.work.weixin.qq.com/document/path/90216

        **权限说明**：
        自建应用或通讯同步助手可以获取所有标签列表；第三方应用仅可获取自己创建的标签。

        返回结果示例： ::

            {
               "errcode": 0,
               "errmsg": "ok",
               "taglist":[
                  {"tagid":1,"tagname":"a"},
                  {"tagid":2,"tagname":"b"}
               ]
            }

        结果参数说明：

        +---------+------------------------+
        | 参数    | 说明                   |
        +=========+========================+
        | errcode | 返回码                 |
        +---------+------------------------+
        | errmsg  | 对返回码的文本描述内容 |
        +---------+------------------------+
        | taglist | 标签列表               |
        +---------+------------------------+
        | tagid   | 标签id                 |
        +---------+------------------------+
        | tagname | 标签名                 |
        +---------+------------------------+

        :return: 标签信息列表，不包含errcode等信息
        """
        res = self._get("tag/list")
        return res["taglist"]



# -*- coding: utf-8 -*-


from wechatpy.exceptions import WeChatException


class InvalidCorpIdException(WeChatException):
    def __init__(self, errcode=-40005, errmsg="Invalid corp_id"):
        super().__init__(errcode, errmsg)



# -*- coding: utf-8 -*-


from wechatpy import replies
from wechatpy.fields import IntegerField, TaskCardField

REPLY_TYPES = {}


def register_reply(reply_type):
    def register(cls):
        REPLY_TYPES[reply_type] = cls
        return cls

    return register


@register_reply("text")
class TextReply(replies.TextReply):
    agent = IntegerField("AgentID", 0)


@register_reply("image")
class ImageReply(replies.ImageReply):
    agent = IntegerField("AgentID", 0)


@register_reply("voice")
class VoiceReply(replies.VoiceReply):
    agent = IntegerField("AgentID", 0)


@register_reply("video")
class VideoReply(replies.VideoReply):
    agent = IntegerField("AgentID", 0)


@register_reply("news")
class ArticlesReply(replies.ArticlesReply):
    agent = IntegerField("AgentID", 0)


@register_reply("update_taskcard")
class TaskCardReply(replies.BaseReply):
    """被动回复格式-任务卡更新消息
    接口详细文档：
    https://work.weixin.qq.com/api/doc/90000/90135/90241#%E4%BB%BB%E5%8A%A1%E5%8D%A1%E7%89%87%E6%9B%B4%E6%96%B0%E6%B6%88%E6%81%AF
    """

    agent = IntegerField("AgentID", 0)
    type = "update_taskcard"
    taskcard = TaskCardField("")

    @property
    def replace_name(self):
        return self.taskcard

    @replace_name.setter
    def replace_name(self, value):
        self.taskcard = value


def create_reply(reply, message=None, render=False):
    r = None
    if isinstance(reply, replies.BaseReply):
        r = reply
        if message:
            r.source = message.target
            r.target = message.source
            r.agent = message.agent
    elif isinstance(reply, str):
        r = TextReply(message=message, content=reply)
    elif isinstance(reply, (tuple, list)):
        if len(reply) > 10:
            raise AttributeError("Can't add more than 10 articles in an ArticlesReply")
        r = ArticlesReply(message=message, articles=reply)
    if r and render:
        return r.render()
    return r



# -*- coding: utf-8 -*-
"""
    wechatpy.fields
    ~~~~~~~~~~~~~~~~

    This module defines some useful field types for parse WeChat messages

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
import time
from datetime import datetime
import base64
import copy
from typing import Any, Callable, Optional

from wechatpy.utils import to_text, to_binary, ObjectDict, timezone


default_timezone = timezone("Asia/Shanghai")


class FieldDescriptor:
    def __init__(self, field):
        self.field = field
        self.attr_name = field.name

    def __get__(self, instance, instance_type=None):
        if instance is not None:
            value = instance._data.get(self.attr_name)
            if value is None:
                value = copy.deepcopy(self.field.default)
                instance._data[self.attr_name] = value
            if isinstance(value, dict):
                value = ObjectDict(value)
            if value and not isinstance(value, (dict, list, tuple)) and callable(self.field.converter):
                value = self.field.converter(value)
            return value
        return self.field

    def __set__(self, instance, value):
        instance._data[self.attr_name] = value


class BaseField:
    converter: Optional[Callable[..., Any]] = None

    def __init__(self, name, default=None):
        self.name = name
        self.default = default

    def to_xml(self, value):
        raise NotImplementedError()

    @classmethod
    def from_xml(cls, value):
        raise NotImplementedError()

    def __getitem__(self, item):
        """有时微信会推嵌套的消息，mypy 的类型检查会愣住，所以加个函数敷衍一下 mypy"""
        raise NotImplementedError()

    def __repr__(self):
        _repr = f"{self.__class__.__name__}({repr(self.name)})"
        return _repr

    def add_to_class(self, klass, name):
        self.klass = klass
        klass._fields[name] = self
        setattr(klass, name, FieldDescriptor(self))


class StringField(BaseField):
    def __to_text(self, value):
        return to_text(value)

    converter = __to_text

    def to_xml(self, value):
        value = self.converter(value)
        return f"<{self.name}><![CDATA[{value}]]></{self.name}>"

    @classmethod
    def from_xml(cls, value):
        return value


class IntegerField(BaseField):
    converter = int

    def to_xml(self, value):
        value = self.converter(value) if value is not None else self.default
        return f"<{self.name}>{value}</{self.name}>"

    @classmethod
    def from_xml(cls, value):
        return cls.converter(value)


class DateTimeField(BaseField):
    def __converter(self, value):
        v = int(value)
        return datetime.fromtimestamp(v, tz=default_timezone)

    converter = __converter

    def to_xml(self, value):
        value = time.mktime(datetime.timetuple(value))
        value = int(value)
        return f"<{self.name}>{value}</{self.name}>"

    @classmethod
    def from_xml(cls, value):
        return cls.converter(None, value)


class FloatField(BaseField):
    converter = float

    def to_xml(self, value):
        value = self.converter(value) if value is not None else self.default
        return f"<{self.name}>{value}</{self.name}>"

    @classmethod
    def from_xml(cls, value):
        return cls.converter(value)


class ImageField(StringField):
    def to_xml(self, value):
        value = self.converter(value)
        return f"""<Image>
        <MediaId><![CDATA[{value}]]></MediaId>
        </Image>"""

    @classmethod
    def from_xml(cls, value):
        return value["MediaId"]


class VoiceField(StringField):
    def to_xml(self, value):
        value = self.converter(value)
        return f"""<Voice>
        <MediaId><![CDATA[{value}]]></MediaId>
        </Voice>"""

    @classmethod
    def from_xml(cls, value):
        return value["MediaId"]


class VideoField(StringField):
    def to_xml(self, value):
        kwargs = dict(media_id=self.converter(value["media_id"]))
        content = "<MediaId><![CDATA[{media_id}]]></MediaId>"
        if "title" in value:
            kwargs["title"] = self.converter(value["title"])
            content += "<Title><![CDATA[{title}]]></Title>"
        if "description" in value:
            kwargs["description"] = self.converter(value["description"])
            content += "<Description><![CDATA[{description}]]></Description>"
        tpl = f"""<Video>{content}</Video>"""
        return tpl.format(**kwargs)

    @classmethod
    def from_xml(cls, value):
        rv = dict(media_id=value["MediaId"])
        if "Title" in value:
            rv["title"] = value["Title"]
        if "Description" in value:
            rv["description"] = value["Description"]
        return rv


class MusicField(StringField):
    def to_xml(self, value):
        kwargs = dict(thumb_media_id=self.converter(value["thumb_media_id"]))
        content = "<ThumbMediaId><![CDATA[{thumb_media_id}]]></ThumbMediaId>"
        if "title" in value:
            kwargs["title"] = self.converter(value["title"])
            content += "<Title><![CDATA[{title}]]></Title>"
        if "description" in value:
            kwargs["description"] = self.converter(value["description"])
            content += "<Description><![CDATA[{description}]]></Description>"
        if "music_url" in value:
            kwargs["music_url"] = self.converter(value["music_url"])
            content += "<MusicUrl><![CDATA[{music_url}]]></MusicUrl>"
        if "hq_music_url" in value:
            kwargs["hq_music_url"] = self.converter(value["hq_music_url"])
            content += "<HQMusicUrl><![CDATA[{hq_music_url}]]></HQMusicUrl>"
        tpl = f"""<Music>{content}</Music>"""
        return tpl.format(**kwargs)

    @classmethod
    def from_xml(cls, value):
        rv = dict(thumb_media_id=value["ThumbMediaId"])
        if "Title" in value:
            rv["title"] = value["Title"]
        if "Description" in value:
            rv["description"] = value["Description"]
        if "MusicUrl" in value:
            rv["music_url"] = value["MusicUrl"]
        if "HQMusicUrl" in value:
            rv["hq_music_url"] = value["HQMusicUrl"]
        return rv


class ArticlesField(StringField):
    def to_xml(self, articles):
        article_count = len(articles)
        items = []
        for article in articles:
            title = self.converter(article.get("title", ""))
            description = self.converter(article.get("description", ""))
            image = self.converter(article.get("image", ""))
            url = self.converter(article.get("url", ""))
            item = f"""<item>
            <Title><![CDATA[{title}]]></Title>
            <Description><![CDATA[{description}]]></Description>
            <PicUrl><![CDATA[{image}]]></PicUrl>
            <Url><![CDATA[{url}]]></Url>
            </item>"""
            items.append(item)
        items_str = "\n".join(items)
        return f"""<ArticleCount>{article_count}</ArticleCount>
        <Articles>{items_str}</Articles>"""

    @classmethod
    def from_xml(cls, value):
        return [
            dict(
                title=item["Title"],
                description=item["Description"],
                image=item["PicUrl"],
                url=item["Url"],
            )
            for item in value["item"]
        ]


class TaskCardField(StringField):
    def to_xml(self, value):
        value = self.converter(value)
        return f"""<TaskCard>
            <ReplaceName><![CDATA[{value}]]></ReplaceName>
        </TaskCard>"""

    @classmethod
    def from_xml(cls, value):
        return value["ReplaceName"]


class Base64EncodeField(StringField):
    def __base64_encode(self, text):
        return to_text(base64.b64encode(to_binary(text)))

    converter = __base64_encode


class Base64DecodeField(StringField):
    def __base64_decode(self, text):
        return to_text(base64.b64decode(to_binary(text)))

    converter = __base64_decode


class HardwareField(StringField):
    def to_xml(self, value=None):
        value = value or {"view": "myrank", "action": "ranklist"}
        return f"""<{self.name}>
        <MessageView><![CDATA[{value.get("view")}]]></MessageView>
        <MessageAction><![CDATA[{value.get("action")}]]></MessageAction>
        </{self.name}>"""



# -*- coding: utf-8 -*-
import logging
import time

from wechatpy.pay.api.base import BaseWeChatPayAPI
from wechatpy.pay.utils import calculate_signature
from wechatpy.utils import random_string, to_text

logger = logging.getLogger(__name__)


class WeChatJSAPI(BaseWeChatPayAPI):
    def get_jsapi_signature(self, prepay_id, timestamp=None, nonce_str=None):
        """
        获取 JSAPI 签名

        :param prepay_id: 统一下单接口返回的 prepay_id 参数值
        :param timestamp: 可选，时间戳，默认为当前时间戳
        :param nonce_str: 可选，随机字符串，默认自动生成
        :return: 签名
        """
        data = {
            "appId": self.sub_appid or self.appid,
            "timeStamp": timestamp or to_text(int(time.time())),
            "nonceStr": nonce_str or random_string(32),
            "signType": "MD5",
            "package": f"prepay_id={prepay_id}",
        }
        return calculate_signature(
            data,
            self._client.api_key if not self._client.sandbox else self._client.sandbox_api_key,
        )

    def get_jsapi_params(self, prepay_id, timestamp=None, nonce_str=None, jssdk=False):
        """
        获取 JSAPI 参数

        :param prepay_id: 统一下单接口返回的 prepay_id 参数值
        :param timestamp: 可选，时间戳，默认为当前时间戳
        :param nonce_str: 可选，随机字符串，默认自动生成
        :param jssdk: 前端调用方式，默认使用 WeixinJSBridge
                      使用 jssdk 调起支付的话，timestamp 的 s 为小写
                      使用 WeixinJSBridge 调起支付的话，timeStamp 的 S 为大写
        :return: 参数
        """
        data = {
            "appId": self.sub_appid or self.appid,
            "timeStamp": timestamp or to_text(int(time.time())),
            "nonceStr": nonce_str or random_string(32),
            "signType": "MD5",
            "package": f"prepay_id={prepay_id}",
        }
        sign = calculate_signature(
            data,
            self._client.api_key if not self._client.sandbox else self._client.sandbox_api_key,
        )
        logger.debug("JSAPI payment parameters: data = %s, sign = %s", data, sign)
        data["paySign"] = sign
        if jssdk:
            data["timestamp"] = data.pop("timeStamp")
        return data



# -*- coding: utf-8 -*-
import json

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMaterial(BaseWeChatAPI):
    def add_drafts(self, articles):
        """
        新增草稿素材, 原 新增永久图文素材
        用于替换原本的新增永久图文素材add_articles方法（接口为add_news）
        详情参考
        https://developers.weixin.qq.com/doc/offiaccount/Draft_Box/Add_draft.html

        :param articles: 图文素材数组
        :type articles: list[dict]
        :return: 返回的 JSON 数据包
        """
        articles_data = []
        for article in articles:
            articles_data.append(
                {
                    "title": article["title"],
                    "author": article.get("author", ""),
                    "digest": article.get("digest", ""),
                    "content": article["content"],
                    "content_source_url": article.get("content_source_url", ""),
                    "thumb_media_id": article["thumb_media_id"],
                    "need_open_comment": int(article.get("need_open_comment", False)),
                    "only_fans_can_comment": int(article.get("only_fans_can_comment", False)),
                }
            )
        return self._post("draft/add", data={"articles": articles_data})

    def submit(self, media_id):
        """
        发布图文消息，前置条件是先将其存为草稿（使用add_drafts方法）
        详情请参考：
        https://developers.weixin.qq.com/doc/offiaccount/Publish/Publish.html
        :param media_id: 图文素材ID
        :type media_id: str
        :return: 返回的 JSON 数据包
        """
        return self._post("freepublish/submit", data={"media_id": media_id})

    def add(self, media_type, media_file, title=None, introduction=None):
        """
        新增其它类型永久素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Adding_Permanent_Assets.html

        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（thumb）
        :param media_file: 要上传的文件，一个 File-object
        :param title: 视频素材标题，仅上传视频素材时需要
        :param introduction: 视频素材简介，仅上传视频素材时需要
        :return: 返回的 JSON 数据包
        """
        params = {"access_token": self.access_token, "type": media_type}
        if media_type == "video":
            assert title, "Video title must be set"
            assert introduction, "Video introduction must be set"
            description = {"title": title, "introduction": introduction}
            params["description"] = json.dumps(description)
        return self._post("material/add_material", params=params, files={"media": media_file})

    def get(self, media_id):
        """
        获取永久素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Getting_Permanent_Assets.html

        :param media_id: 素材的 media_id
        :return: 图文素材返回图文列表，其它类型为素材的内容
        """

        def _processor(res):
            if isinstance(res, dict) and "news_item" in res:
                # 图文素材
                return res["news_item"]
            return res

        return self._post("material/get_material", data={"media_id": media_id}, result_processor=_processor)

    def delete(self, media_id):
        """
        删除永久素材
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Deleting_Permanent_Assets.html

        :param media_id: 素材的 media_id
        :return: 返回的 JSON 数据包
        """
        return self._post("material/del_material", data={"media_id": media_id})

    def batchget(self, media_type, offset=0, count=20):
        """
        获取素材列表
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Get_materials_list.html

        :param media_type: 媒体文件类型，分别有图片（image）、语音（voice）、视频（video）和缩略图（news）
        :param offset: 从全部素材的该偏移位置开始返回，0 表示从第一个素材返回
        :param count: 返回素材的数量，取值在1到20之间
        :return: 返回的 JSON 数据包
        """
        return self._post("material/batchget_material", data={"type": media_type, "offset": offset, "count": count})

    def get_count(self):
        """
        获取素材总数
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Asset_Management/Get_the_total_of_all_materials.html

        :return: 返回的 JSON 数据包
        """
        return self._get("material/get_materialcount")

    def open_comment(self, msg_data_id, index=1):
        """
        打开已群发文章评论
        https://mp.weixin.qq.com/wiki?id=mp1494572718_WzHIY
        """
        return self._post("comment/open", data={"msg_data_id": msg_data_id, "index": index})

    def close_comment(self, msg_data_id, index=1):
        """
        关闭已群发文章评论
        """
        return self._post("comment/close", data={"msg_data_id": msg_data_id, "index": index})

    def list_comment(self, msg_data_id, index=1, begin=0, count=50, type=0):
        """
        查看指定文章的评论数据
        """
        return self._post(
            "comment/list",
            data={"msg_data_id": msg_data_id, "index": index, "begin": begin, "count": count, "type": type},
        )

    def markelect_comment(self, msg_data_id, index, user_comment_id):
        """
        将评论标记精选
        """
        return self._post(
            "comment/markelect",
            data={"msg_data_id": msg_data_id, "index": index, "user_comment_id": user_comment_id},
        )

    def unmarkelect_comment(self, msg_data_id, index, user_comment_id):
        """
        将评论取消精选
        """
        return self._post(
            "comment/unmarkelect",
            data={"msg_data_id": msg_data_id, "index": index, "user_comment_id": user_comment_id},
        )

    def delete_comment(self, msg_data_id, index, user_comment_id):
        """
        删除评论
        """
        return self._post(
            "comment/delete",
            data={"msg_data_id": msg_data_id, "index": index, "user_comment_id": user_comment_id},
        )

    def add_reply_comment(self, msg_data_id, index, user_comment_id, content):
        """
        回复评论
        """
        return self._post(
            "comment/reply/add",
            data={"msg_data_id": msg_data_id, "index": index, "user_comment_id": user_comment_id, "content": content},
        )

    def delete_reply_comment(self, msg_data_id, index, user_comment_id):
        """
        删除回复
        """
        return self._post(
            "comment/reply/delete",
            data={"msg_data_id": msg_data_id, "index": index, "user_comment_id": user_comment_id},
        )



# -*- coding: utf-8 -*-
"""
    wechatpy.replies
    ~~~~~~~~~~~~~~~~~~
    This module defines all kinds of replies you can send to WeChat

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""

import time
import xmltodict

from wechatpy.fields import (
    StringField,
    IntegerField,
    ImageField,
    VoiceField,
    VideoField,
    MusicField,
    ArticlesField,
    Base64EncodeField,
    HardwareField,
)
from wechatpy.messages import BaseMessage, MessageMetaClass


REPLY_TYPES = {}


def register_reply(reply_type):
    def register(cls):
        REPLY_TYPES[reply_type] = cls
        return cls

    return register


class BaseReply(metaclass=MessageMetaClass):
    """Base class for all replies"""

    source = StringField("FromUserName")
    target = StringField("ToUserName")
    time = IntegerField("CreateTime", time.time())
    type = "unknown"

    def __init__(self, **kwargs):
        self._data = {}
        message = kwargs.pop("message", None)
        if message and isinstance(message, BaseMessage):
            if "source" not in kwargs:
                kwargs["source"] = message.target
            if "target" not in kwargs:
                kwargs["target"] = message.source
            if hasattr(message, "agent") and "agent" not in kwargs:
                kwargs["agent"] = message.agent
        if "time" not in kwargs:
            kwargs["time"] = time.time()
        for name, value in kwargs.items():
            field = self._fields.get(name)
            if field:
                self._data[field.name] = value
            else:
                setattr(self, name, value)

    def render(self):
        """Render reply from Python object to XML string"""
        nodes = []
        msg_type = f"<MsgType><![CDATA[{self.type}]]></MsgType>"
        nodes.append(msg_type)
        for name, field in self._fields.items():
            value = getattr(self, name, field.default)
            node_xml = field.to_xml(value)
            nodes.append(node_xml)
        data = "\n".join(nodes)
        return f"<xml>\n{data}\n</xml>"

    def __str__(self):
        return self.render()


@register_reply("empty")
class EmptyReply(BaseReply):
    """
    回复空串

    微信服务器不会对此作任何处理，并且不会发起重试
    """

    def __init__(self):
        pass

    def render(self):
        return ""


@register_reply("text")
class TextReply(BaseReply):
    """
    文本回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "text"
    content = StringField("Content")


@register_reply("image")
class ImageReply(BaseReply):
    """
    图片回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "image"
    image = ImageField("Image")

    @property
    def media_id(self):
        return self.image

    @media_id.setter
    def media_id(self, value):
        self.image = value


@register_reply("voice")
class VoiceReply(BaseReply):
    """
    语音回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "voice"
    voice = VoiceField("Voice")

    @property
    def media_id(self):
        return self.voice

    @media_id.setter
    def media_id(self, value):
        self.voice = value


@register_reply("video")
class VideoReply(BaseReply):
    """
    视频回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "video"
    video = VideoField("Video", {})

    @property
    def media_id(self):
        return self.video.get("media_id")

    @media_id.setter
    def media_id(self, value):
        video = self.video
        video["media_id"] = value
        self.video = video

    @property
    def title(self):
        return self.video.get("title")

    @title.setter
    def title(self, value):
        video = self.video
        video["title"] = value
        self.video = video

    @property
    def description(self):
        return self.video.get("description")

    @description.setter
    def description(self, value):
        video = self.video
        video["description"] = value
        self.video = video


@register_reply("music")
class MusicReply(BaseReply):
    """
    音乐回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "music"
    music = MusicField("Music", {})

    @property
    def thumb_media_id(self):
        return self.music.get("thumb_media_id")

    @thumb_media_id.setter
    def thumb_media_id(self, value):
        music = self.music
        music["thumb_media_id"] = value
        self.music = music

    @property
    def title(self):
        return self.music.get("title")

    @title.setter
    def title(self, value):
        music = self.music
        music["title"] = value
        self.music = music

    @property
    def description(self):
        return self.music.get("description")

    @description.setter
    def description(self, value):
        music = self.music
        music["description"] = value
        self.music = music

    @property
    def music_url(self):
        return self.music.get("music_url")

    @music_url.setter
    def music_url(self, value):
        music = self.music
        music["music_url"] = value
        self.music = music

    @property
    def hq_music_url(self):
        return self.music.get("hq_music_url")

    @hq_music_url.setter
    def hq_music_url(self, value):
        music = self.music
        music["hq_music_url"] = value
        self.music = music


@register_reply("news")
class ArticlesReply(BaseReply):
    """
    图文回复
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Message_Management/Passive_user_reply_message.html
    """

    type = "news"
    articles = ArticlesField("Articles", [])

    def add_article(self, article):
        if len(self.articles) == 10:
            raise AttributeError("Can't add more than 10 articles in an ArticlesReply")
        articles = self.articles
        articles.append(article)
        self.articles = articles


@register_reply("transfer_customer_service")
class TransferCustomerServiceReply(BaseReply):
    """
    将消息转发到多客服
    详情请参阅
    https://developers.weixin.qq.com/doc/offiaccount/Customer_Service/Forwarding_of_messages_to_service_center.html
    """

    type = "transfer_customer_service"


@register_reply("device_text")
class DeviceTextReply(BaseReply):
    type = "device_text"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    session_id = StringField("SessionID")
    content = Base64EncodeField("Content")


@register_reply("device_event")
class DeviceEventReply(BaseReply):
    type = "device_event"
    event = StringField("Event")
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    session_id = StringField("SessionID")
    content = Base64EncodeField("Content")


@register_reply("device_status")
class DeviceStatusReply(BaseReply):
    type = "device_status"
    device_type = StringField("DeviceType")
    device_id = StringField("DeviceID")
    status = IntegerField("DeviceStatus")


@register_reply("hardware")
class HardwareReply(BaseReply):
    type = "hardware"
    func_flag = IntegerField("FuncFlag", 0)
    hardware = HardwareField("HardWare")


def create_reply(reply, message=None, render=False):
    """
    Create a reply quickly
    """
    r = None
    if not reply:
        r = EmptyReply()
    elif isinstance(reply, BaseReply):
        r = reply
        if message:
            r.source = message.target
            r.target = message.source
    elif isinstance(reply, str):
        r = TextReply(message=message, content=reply)
    elif isinstance(reply, (tuple, list)):
        if len(reply) > 10:
            raise AttributeError("Can't add more than 10 articles in an ArticlesReply")
        r = ArticlesReply(message=message, articles=reply)
    if r and render:
        return r.render()
    return r


def deserialize_reply(xml, update_time=False):
    """
    反序列化被动回复
    :param xml: 待反序列化的xml
    :param update_time: 是否用当前时间替换xml中的时间
    :raises ValueError: 不能辨识的reply xml
    :rtype: wechatpy.replies.BaseReply
    """
    if not xml:
        return EmptyReply()

    try:
        reply_dict = xmltodict.parse(xml)["xml"]
        msg_type = reply_dict["MsgType"]
    except (xmltodict.expat.ExpatError, KeyError):
        raise ValueError("bad reply xml")
    if msg_type not in REPLY_TYPES:
        raise ValueError("unknown reply type")

    cls = REPLY_TYPES[msg_type]
    kwargs = {}
    for attr, field in cls._fields.items():
        if field.name in reply_dict:
            str_value = reply_dict[field.name]
            kwargs[attr] = field.from_xml(str_value)

    if update_time:
        kwargs["time"] = time.time()

    return cls(**kwargs)



# -*- coding: utf-8 -*-

from wechatpy.pay.api.redpack import WeChatRedpack  # NOQA
from wechatpy.pay.api.transfer import WeChatTransfer  # NOQA
from wechatpy.pay.api.coupon import WeChatCoupon  # NOQA
from wechatpy.pay.api.order import WeChatOrder  # NOQA
from wechatpy.pay.api.refund import WeChatRefund  # NOQA
from wechatpy.pay.api.tools import WeChatTools  # NOQA
from wechatpy.pay.api.jsapi import WeChatJSAPI  # NOQA
from wechatpy.pay.api.micropay import WeChatMicroPay  # NOQA
from wechatpy.pay.api.withhold import WeChatWithhold  # NOQA
from wechatpy.pay.api.appauth import WeChatAppAuth  # NOQA



# -*- coding: utf-8 -*-

import random
from datetime import datetime

from wechatpy.pay.utils import get_external_ip, rsa_encrypt
from wechatpy.pay.api.base import BaseWeChatPayAPI


class WeChatTransfer(BaseWeChatPayAPI):
    def transfer(
        self,
        user_id,
        amount,
        desc,
        client_ip=None,
        check_name="OPTION_CHECK",
        real_name=None,
        out_trade_no=None,
        device_info=None,
    ):
        """
        企业付款接口

        :param user_id: 接受收红包的用户在公众号下的 openid
        :param amount: 付款金额，单位分
        :param desc: 付款说明
        :param client_ip: 可选，调用接口机器的 IP 地址
        :param check_name: 可选，校验用户姓名选项，
                           NO_CHECK：不校验真实姓名,
                           FORCE_CHECK：强校验真实姓名（未实名认证的用户会校验失败，无法转账）,
                           OPTION_CHECK：针对已实名认证的用户才校验真实姓名（未实名认证用户不校验，可以转账成功）,
                           默认为 OPTION_CHECK
        :param real_name: 可选，收款用户真实姓名，
                          如果check_name设置为FORCE_CHECK或OPTION_CHECK，则必填用户真实姓名
        :param out_trade_no: 可选，商户订单号，需保持唯一性，默认自动生成
        :param device_info: 可选，微信支付分配的终端设备号
        :return: 返回的结果信息
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "mch_appid": self.appid,
            "mchid": self.mch_id,
            "device_info": device_info,
            "partner_trade_no": out_trade_no,
            "openid": user_id,
            "check_name": check_name,
            "re_user_name": real_name,
            "amount": amount,
            "desc": desc,
            "spbill_create_ip": client_ip or get_external_ip(),
        }
        return self._post("mmpaymkttransfers/promotion/transfers", data=data)

    def query(self, out_trade_no):
        """
        企业付款查询接口

        :param out_trade_no: 商户调用企业付款API时使用的商户订单号
        :return: 返回的结果数据
        """
        data = {
            "appid": self.appid,
            "partner_trade_no": out_trade_no,
        }
        return self._post("mmpaymkttransfers/gettransferinfo", data=data)

    def transfer_bankcard(self, true_name, bank_card_no, bank_code, amount, desc=None, out_trade_no=None):
        """
        企业付款到银行卡接口

        :param true_name: 开户人名称
        :param bank_card_no: 银行卡号
        :param bank_code: 银行编号
        :param amount: 付款金额，单位分
        :param desc: 付款说明
        :param out_trade_no: 可选，商户订单号，需保持唯一性，默认自动生成
        :return: 返回的结果信息
        """
        if not out_trade_no:
            now = datetime.now()
            out_trade_no = f"{self.mch_id}{now.strftime('%Y%m%d%H%M%S')}{random.randint(1000, 10000)}"
        data = {
            "mch_id": self.mch_id,
            "partner_trade_no": out_trade_no,
            "amount": amount,
            "desc": desc,
            "enc_bank_no": self._rsa_encrypt(bank_card_no),
            "enc_true_name": self._rsa_encrypt(true_name),
            "bank_code": bank_code,
        }
        return self._post("mmpaysptrans/pay_bank", data=data)

    def query_bankcard(self, out_trade_no):
        """
        企业付款查询接口

        :param out_trade_no: 商户调用企业付款API时使用的商户订单号
        :return: 返回的结果数据
        """
        data = {
            "mch_id": self.mch_id,
            "partner_trade_no": out_trade_no,
        }
        return self._post("mmpaysptrans/query_bank", data=data)

    def get_rsa_public_key(self):
        data = {
            "mch_id": self.mch_id,
            "sign_type": "MD5",
        }
        return self._post("https://fraud.mch.weixin.qq.com/risk/getpublickey", data=data)

    def _rsa_encrypt(self, data):
        if not getattr(self, "_rsa_public_key", None):
            self._rsa_public_key = self.get_rsa_public_key()["pub_key"]
        return rsa_encrypt(data, self._rsa_public_key)



# -*- coding: utf-8 -*-


from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMisc(BaseWeChatAPI):
    def get_wechat_ips(self):
        """
        获取企业微信服务器的ip段

        https://work.weixin.qq.com/api/doc#90000/90135/90238/%E8%8E%B7%E5%8F%96%E4%BC%81%E4%B8%9A%E5%BE%AE%E4%BF%A1%E6%9C%8D%E5%8A%A1%E5%99%A8%E7%9A%84ip%E6%AE%B5

        :return: 企业微信回调的IP段
        """
        res = self._get("getcallbackip")
        return res["ip_list"]



# -*- coding: utf-8 -*-

from datetime import datetime, date

from optionaldict import optionaldict
from wechatpy.client.api.base import BaseWeChatAPI


class WeChatWiFi(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/bizwifi/"

    def list_shops(self, page_index=1, page_size=20):
        """
        获取门店列表

        详情请参考
        http://mp.weixin.qq.com/wiki/15/bcfb5d4578ea818b89913472cf2bbf8f.html

        :param page_index: 可选，分页下标，默认从1开始
        :param page_size: 可选，每页的个数，默认20个，最大20个
        :return: 返回的 JSON 数据包
        """
        res = self._post(
            "shop/list",
            data={
                "pageindex": page_index,
                "pagesize": page_size,
            },
            result_processor=lambda x: x["data"],
        )
        return res

    def get_shop(self, shop_id=0):
        """
        查询门店的WiFi信息
        http://mp.weixin.qq.com/wiki/15/bcfb5d4578ea818b89913472cf2bbf8f.html

        :param shop_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        res = self._post(
            "shop/get",
            data={
                "shop_id": shop_id,
            },
            result_processor=lambda x: x["data"],
        )
        return res

    def add_device(self, shop_id, ssid, password, bssid):
        """
        添加设备

        详情请参考
        http://mp.weixin.qq.com/wiki/10/6232005bdc497f7cf8e19d4e843c70d2.html

        :param shop_id: 门店 ID
        :param ssid: 无线网络设备的ssid。非认证公众号添加的ssid必需是“WX”开头(“WX”为大写字母)，
                     认证公众号和第三方平台无此限制；所有ssid均不能包含中文字符
        :param password: 无线网络设备的密码，大于8个字符，不能包含中文字符
        :param bssid: 无线网络设备无线mac地址，格式冒号分隔，字符长度17个，并且字母小写
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "device/add",
            data={
                "shop_id": shop_id,
                "ssid": ssid,
                "password": password,
                "bssid": bssid,
            },
        )

    def list_devices(self, shop_id=None, page_index=1, page_size=20):
        """
        查询设备

        详情请参考
        http://mp.weixin.qq.com/wiki/10/6232005bdc497f7cf8e19d4e843c70d2.html

        :param shop_id: 可选，门店 ID
        :param page_index: 可选，分页下标，默认从1开始
        :param page_size: 可选，每页的个数，默认20个，最大20个
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(shop_id=shop_id, pageindex=page_index, pagesize=page_size)
        res = self._post("device/list", data=data, result_processor=lambda x: x["data"])
        return res

    def delete_device(self, bssid):
        """
        删除设备

        详情请参考
        http://mp.weixin.qq.com/wiki/10/6232005bdc497f7cf8e19d4e843c70d2.html

        :param bssid: 无线网络设备无线mac地址，格式冒号分隔，字符长度17个，并且字母小写
        :return: 返回的 JSON 数据包
        """
        return self._post("device/delete", data={"bssid": bssid})

    def get_qrcode_url(self, shop_id, img_id):
        """
        获取物料二维码图片网址

        详情请参考
        http://mp.weixin.qq.com/wiki/7/fcd0378ef00617fc276be2b3baa80973.html

        :param shop_id: 门店 ID
        :param img_id: 物料样式编号：0-二维码，可用于自由设计宣传材料；
                       1-桌贴（二维码），100mm×100mm(宽×高)，可直接张贴
        :return: 二维码图片网址
        """
        res = self._post(
            "qrcode/get",
            data={
                "shop_id": shop_id,
                "img_id": img_id,
            },
            result_processor=lambda x: x["data"]["qrcode_url"],
        )
        return res

    def set_homepage(self, shop_id, template_id, url=None):
        """
        设置商家主页

        详情请参考
        http://mp.weixin.qq.com/wiki/6/2732f3cf83947e0e4971aa8797ee9d6a.html

        :param shop_id: 门店 ID
        :param template_id: 模板ID，0-默认模板，1-自定义url
        :param url: 自定义链接，当template_id为1时必填
        :return: 返回的 JSON 数据包
        """
        data = {
            "shop_id": shop_id,
            "template_id": template_id,
        }
        if url:
            data["struct"] = {"url": url}
        return self._post("homepage/set", data=data)

    def get_homepage(self, shop_id):
        """
        查询商家主页

        详情请参考
        http://mp.weixin.qq.com/wiki/6/2732f3cf83947e0e4971aa8797ee9d6a.html

        :param shop_id: 门店 ID
        :return: 返回的 JSON 数据包
        """
        res = self._post(
            "homepage/get",
            data={"shop_id": shop_id},
            result_processor=lambda x: x["data"],
        )
        return res

    def list_statistics(self, begin_date, end_date, shop_id=-1):
        """
        Wi-Fi数据统计

        详情请参考
        http://mp.weixin.qq.com/wiki/8/dfa2b756b66fca5d9b1211bc18812698.html

        :param begin_date: 起始日期时间，最长时间跨度为30天
        :param end_date: 结束日期时间戳，最长时间跨度为30天
        :param shop_id: 可选，门店 ID，按门店ID搜索，-1为总统计
        :return: 返回的 JSON 数据包
        """
        if isinstance(begin_date, (datetime, date)):
            begin_date = begin_date.strftime("%Y-%m-%d")
        if isinstance(end_date, (datetime, date)):
            end_date = end_date.strftime("%Y-%m-%d")
        res = self._post(
            "statistics/list",
            data={"begin_date": begin_date, "end_date": end_date, "shop_id": shop_id},
            result_processor=lambda x: x["data"],
        )
        return res



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI

from wechatpy.client.api.merchant.category import MerchantCategory
from wechatpy.client.api.merchant.stock import MerchantStock
from wechatpy.client.api.merchant.express import MerchantExpress
from wechatpy.client.api.merchant.group import MerchantGroup
from wechatpy.client.api.merchant.shelf import MerchantShelf
from wechatpy.client.api.merchant.order import MerchantOrder
from wechatpy.client.api.merchant.common import MerchantCommon


class WeChatMerchant(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/"

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        # sub APIs
        self.category = MerchantCategory(self._client)
        self.stock = MerchantStock(self._client)
        self.express = MerchantExpress(self._client)
        self.group = MerchantGroup(self._client)
        self.shelf = MerchantShelf(self._client)
        self.order = MerchantOrder(self._client)
        self.common = MerchantCommon(self._client)

    def create(self, product_data):
        """增加商品"""
        return self._post("merchant/create", data=product_data)

    def delete(self, product_id):
        """删除商品"""
        return self._post("merchant/del", data={"product_id": product_id})

    def update(self, product_id, product_data):
        """修改商品"""
        product_data["product_id"] = product_id
        return self._post("merchant/update", data=product_data)

    def get(self, product_id):
        """查询商品"""
        return self._post("merchant/get", data={"product_id": product_id})

    def get_by_status(self, status):
        """获取指定状态的所有商品"""
        return self._post("merchant/getbystatus", data={"status": status})

    def update_product_status(self, product_id, status):
        """商品上下架"""
        return self._post(
            "merchant/modproductstatus",
            data={"product_id": product_id, "status": status},
        )

    def get_subcategories(self, cate_id):
        """
        获取指定分类的所有子分类

        :param cate_id: 大分类ID(根节点分类id为1)
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/category/getsub", data={"cate_id": cate_id})

    def get_category_sku(self, cate_id):
        """
        获取指定子分类的所有SKU

        :param cate_id: 商品子分类ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/category/getsku", data={"cate_id": cate_id})

    def get_category_property(self, cate_id):
        """
        获取指定分类的所有属性

        :param cate_id: 商品子分类ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/category/getproperty", data={"cate_id": cate_id})

    def add_stock(self, product_id, sku_info, quantity):
        """
        增加库存

        :param product_id: 商品ID
        :param sku_info: sku信息,格式"id1:vid1;id2:vid2",如商品为统一规格，则此处赋值为空字符串即可
        :param quantity: 增加的库存数量
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "merchant/stock/add",
            data={"product_id": product_id, "sku_info": sku_info, "quantity": quantity},
        )

    def reduce_stock(self, product_id, sku_info, quantity):
        """
        减少库存

        :param product_id: 商品ID
        :param sku_info: sku信息,格式"id1:vid1;id2:vid2",如商品为统一规格，则此处赋值为空字符串即可
        :param quantity: 减少的库存数量
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "merchant/stock/reduce",
            data={"product_id": product_id, "sku_info": sku_info, "quantity": quantity},
        )

    def add_express(self, product_data):
        """
        增加邮费模板

        :param product_data: 邮费信息
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/express/add", data=product_data)

    def del_express(self, template_id):
        """
        增加邮费模板

        :param template_id: 邮费模板ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/express/del", data={"template_id": template_id})

    def update_express(self, template_id, delivery_template):
        """
        增加邮费模板

        :param template_id: 邮费模板ID
        :param delivery_template: 邮费模板信息(字段说明详见增加邮费模板)
        :return: 返回的 JSON 数据包
        """
        delivery_template["template_id"] = template_id
        return self._post("merchant/express/update", data=delivery_template)

    def get_express(self, template_id):
        """
        获取指定ID的邮费模板

        :param template_id: 邮费模板ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/express/getbyid", data={"template_id": template_id})

    def get_all_express(self):
        """
        获取所有邮费模板

        :param template_id: 邮费模板ID
        :return: 返回的 JSON 数据包
        """
        return self._get("merchant/express/getall")

    def add_group(self, group_detail):
        """
        增加分组

        :param group_detail: 商品分组信息
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/group/add", data=group_detail)

    def del_group(self, group_id):
        """
        删除分组

        :param group_id: 商品分组ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/group/del", data={"group_id": group_id})

    def update_group_property(self, group_id, group_properties):
        """
        修改分组属性

        :param group_id: 商品分组ID
        :param group_properties: 商品分组属性
        :return: 返回的 JSON 数据包
        """
        group_properties["group_id"] = group_id
        return self._post("merchant/group/propertymod", data=group_properties)

    def update_group_product(self, group_id, product_data):
        """
        修改分组商品

        :param group_id: 商品分组ID
        :param product_data: 分组商品信息
        :return: 返回的 JSON 数据包
        """
        product_data["group_id"] = group_id
        return self._post("merchant/group/productmod", data=product_data)

    def get_all_groups(self):
        """
        获取所有分组

        :return: 返回的 JSON 数据包
        """
        return self._get("merchant/group/getall")

    def get_group(self, group_id):
        """
        根据分组ID获取分组信息

        :param group_id: 商品分组ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/group/getbyid", data={"group_id": group_id})

    def add_shelf(self, shelf_data):
        """
        增加货架

        :param shelf_data: 货架详情信息
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/shelf/add", data=shelf_data)

    def del_shelf(self, shelf_id):
        """
        删除货架

        :param shelf_id: 货架ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/shelf/del", data={"shelf_id": shelf_id})

    def update_shelf(self, shelf_id, shelf_data):
        """
        修改货架

        :param shelf_id: 货架ID
        :param shelf_data: 货架详情
        :return: 返回的 JSON 数据包
        """
        shelf_data["shelf_id"] = shelf_id
        return self._post("merchant/shelf/mod", data=shelf_data)

    def get_all_shelves(self):
        """
        获取所有货架

        :return: 返回的 JSON 数据包
        """
        return self._get("merchant/shelf/getall")

    def get_shelf(self, shelf_id):
        """
        根据货架ID获取货架信息

        :param shelf_id: 货架ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/shelf/getbyid", data={"shelf_id": shelf_id})

    def get_order(self, order_id):
        """
        根据订单ID获取订单详情

        :param order_id: 订单ID
        :return: 返回的 JSON 数据包
        """
        return self._post("merchant/order/getbyid", data={"order_id": order_id})

    def query_order(self, status=None, begintime=None, endtime=None):
        """
        根据订单状态/创建时间获取订单详情

        :param status: 订单状态(不带该字段-全部状态, 2-待发货, 3-已发货, 5-已完成, 8-维权中, )
        :param begintime: 订单创建时间起始时间(不带该字段则不按照时间做筛选)
        :param endtime: 订单创建时间终止时间(不带该字段则不按照时间做筛选)
        :return: 返回的 JSON 数据包
        """
        return self._post(
            "merchant/order/getbyfilter",
            data={"status": status, "begintime": begintime, "endtime": endtime},
        )

    def set_delivery(self, order_id, delivery_data):
        """
        修改货架

        :param order_id: 订单ID
        :param delivery_data: 商品物流信息
        :return: 返回的 JSON 数据包
        """
        delivery_data["order_id"] = order_id
        return self._post("merchant/shelf/setdeliverymod", data=delivery_data)

    def upload_image(self, media_file):
        """
        上传图片

        :param media_file: 要上传的文件，一个 File-object
        :return: 上传成功时返回图片 URL
        """
        res = self._post(
            url="merchant/common/upload_img",
            files={"media": media_file},
            result_processor=lambda x: x["url"],
        )
        return res



# -*- coding: utf-8 -*-


import base64
import copy
import hashlib
import hmac
import socket
import logging

from wechatpy.utils import to_binary, to_text

logger = logging.getLogger(__name__)


def format_url(params, api_key=None):
    data = [to_binary(f"{k}={params[k]}") for k in sorted(params) if params[k]]
    if api_key:
        data.append(to_binary(f"key={api_key}"))
    return b"&".join(data)


def calculate_signature(params, api_key):
    url = format_url(params, api_key)
    logger.debug("Calculate Signature URL: %s", url)
    return to_text(hashlib.md5(url).hexdigest().upper())


def calculate_signature_hmac(params, api_key):
    url = format_url(params, api_key)
    sign = to_text(hmac.new(api_key.encode(), msg=url, digestmod=hashlib.sha256).hexdigest().upper())
    return sign


def _check_signature(params, api_key):
    _params = copy.deepcopy(params)
    sign = _params.pop("sign", "")
    return sign == calculate_signature(_params, api_key)


def dict_to_xml(d, sign=None):
    xml = ["<xml>\n"]
    for k in sorted(d):
        # use sorted to avoid test error on Py3k
        v = d[k]
        if isinstance(v, int) or (isinstance(v, str) and v.isdigit()):
            xml.append(f"<{to_text(k)}>{to_text(v)}</{to_text(k)}>\n")
        else:
            xml.append(f"<{to_text(k)}><![CDATA[{to_text(v)}]]></{to_text(k)}>\n")
    if sign:
        xml.append(f"<sign><![CDATA[{to_text(sign)}]]></sign>\n</xml>")
    else:
        xml.append("</xml>")
    return "".join(xml)


def get_external_ip():
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    try:
        wechat_ip = socket.gethostbyname("api.mch.weixin.qq.com")
        sock.connect((wechat_ip, 80))
        addr, port = sock.getsockname()
        sock.close()
        return addr
    except socket.error:
        return "127.0.0.1"


def rsa_encrypt(data, pem, b64_encode=True):
    """
    rsa 加密
    :param data: 待加密字符串/binary
    :param pem: RSA public key 内容/binary
    :param b64_encode: 是否对输出进行 base64 encode
    :return: 如果 b64_encode=True 的话，返回加密并 base64 处理后的 string；否则返回加密后的 binary
    """
    from cryptography.hazmat.primitives import serialization
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives.asymmetric import padding

    encoded_data = to_binary(data)
    pem = to_binary(pem)
    public_key = serialization.load_pem_public_key(pem)
    encrypted_data = public_key.encrypt(
        encoded_data,
        padding=padding.OAEP(
            mgf=padding.MGF1(hashes.SHA1()),
            algorithm=hashes.SHA1(),
            label=None,
        ),
    )
    if b64_encode:
        encrypted_data = base64.b64encode(encrypted_data).decode("utf-8")
    return encrypted_data


def rsa_decrypt(encrypted_data, pem, password=None):
    """
    rsa 解密
    :param encrypted_data: 待解密 bytes
    :param pem: RSA private key 内容/binary
    :param password: RSA private key pass phrase
    :return: 解密后的 binary
    """
    from cryptography.hazmat.primitives import serialization
    from cryptography.hazmat.primitives import hashes
    from cryptography.hazmat.primitives.asymmetric import padding

    encrypted_data = to_binary(encrypted_data)
    pem = to_binary(pem)
    private_key = serialization.load_pem_private_key(pem, password)
    data = private_key.decrypt(
        encrypted_data,
        padding=padding.OAEP(
            mgf=padding.MGF1(hashes.SHA1()),
            algorithm=hashes.SHA1(),
            label=None,
        ),
    )
    return data



# -*- coding: utf-8 -*-
"""
    wechatpy.component
    ~~~~~~~~~~~~~~~

    This module provides client library for WeChat Open Platform

    :copyright: (c) 2015 by hunter007.
    :license: MIT, see LICENSE for more details.
"""
import json
import logging
import time
from urllib.parse import quote

import requests
import xmltodict

from wechatpy.client import WeChatComponentClient
from wechatpy.constants import WeChatErrorCode
from wechatpy.crypto import WeChatCrypto
from wechatpy.exceptions import (
    APILimitedException,
    WeChatClientException,
    WeChatComponentOAuthException,
    WeChatOAuthException,
)
from wechatpy.messages import COMPONENT_MESSAGE_TYPES, ComponentUnknownMessage
from wechatpy.session.memorystorage import MemoryStorage
from wechatpy.utils import to_text

logger = logging.getLogger(__name__)


class BaseWeChatComponent:
    API_BASE_URL = "https://api.weixin.qq.com/cgi-bin"

    def __init__(
        self,
        component_appid,
        component_appsecret,
        component_token,
        encoding_aes_key,
        session=None,
        auto_retry=True,
    ):
        """
        :param component_appid: 第三方平台appid
        :param component_appsecret: 第三方平台appsecret
        :param component_token: 公众号消息校验Token
        :param encoding_aes_key: 公众号消息加解密Key
        """
        self._http = requests.Session()
        self.component_appid = component_appid
        self.component_appsecret = component_appsecret
        self.expires_at = None
        self.crypto = WeChatCrypto(component_token, encoding_aes_key, component_appid)
        self.session = session or MemoryStorage()
        self.auto_retry = auto_retry

    @property
    def component_verify_ticket(self):
        return self.session.get("{}_component_verify_ticket".format(self.component_appid))

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            api_base_url = kwargs.pop("api_base_url", self.API_BASE_URL)
            url = f"{api_base_url}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if "params" not in kwargs:
            kwargs["params"] = {}
        if isinstance(kwargs["params"], dict) and "component_access_token" not in kwargs["params"]:
            kwargs["params"]["component_access_token"] = self.access_token
        if isinstance(kwargs["data"], dict):
            kwargs["data"] = json.dumps(kwargs["data"])

        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )

        return self._handle_result(res, method, url, **kwargs)

    def _handle_result(self, res, method=None, url=None, **kwargs):
        result = json.loads(res.content.decode("utf-8", "ignore"), strict=False)
        if "errcode" in result:
            result["errcode"] = int(result["errcode"])

        if "errcode" in result and result["errcode"] != 0:
            errcode = result["errcode"]
            errmsg = result.get("errmsg", errcode)
            if self.auto_retry and errcode in (
                WeChatErrorCode.INVALID_CREDENTIAL.value,
                WeChatErrorCode.INVALID_ACCESS_TOKEN.value,
                WeChatErrorCode.EXPIRED_ACCESS_TOKEN.value,
            ):
                logger.info("Component access token expired, fetch a new one and retry request")
                self.fetch_access_token()
                kwargs["params"]["component_access_token"] = self.session.get(
                    "{}_component_access_token".format(self.component_appid)
                )
                return self._request(method=method, url_or_endpoint=url, **kwargs)
            elif errcode == WeChatErrorCode.OUT_OF_API_FREQ_LIMIT.value:
                # api freq out of limit
                raise APILimitedException(errcode, errmsg, client=self, request=res.request, response=res)
            else:
                raise WeChatClientException(errcode, errmsg, client=self, request=res.request, response=res)
        return result

    def fetch_access_token(self):
        """
        获取 component_access_token
        详情请参考 https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list\
        &t=resource/res_list&verify=1&id=open1419318587&token=&lang=zh_CN

        :return: 返回的 JSON 数据包
        """
        url = f"{self.API_BASE_URL}{'/component/api_component_token'}"
        return self._fetch_access_token(
            url=url,
            data=json.dumps(
                {
                    "component_appid": self.component_appid,
                    "component_appsecret": self.component_appsecret,
                    "component_verify_ticket": self.component_verify_ticket,
                }
            ),
        )

    def _fetch_access_token(self, url, data):
        """The real fetch access token"""
        logger.info("Fetching component access token")
        res = self._http.post(url=url, data=data)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )
        result = res.json()
        if "errcode" in result and result["errcode"] != 0:
            raise WeChatClientException(
                result["errcode"],
                result["errmsg"],
                client=self,
                request=res.request,
                response=res,
            )

        expires_in = 7200
        if "expires_in" in result:
            expires_in = result["expires_in"]
        self.session.set(
            "{}_component_access_token".format(self.component_appid), result["component_access_token"], expires_in
        )
        self.expires_at = int(time.time()) + expires_in
        return result

    @property
    def access_token(self):
        """WeChat component access token"""
        access_token = self.session.get("{}_component_access_token".format(self.component_appid))
        if access_token:
            if not self.expires_at:
                # user provided access_token, just return it
                return access_token

            timestamp = time.time()
            if self.expires_at - timestamp > 60:
                return access_token

        self.fetch_access_token()
        return self.session.get("{}_component_access_token".format(self.component_appid))

    def get(self, url, **kwargs):
        return self._request(method="get", url_or_endpoint=url, **kwargs)

    def post(self, url, **kwargs):
        return self._request(method="post", url_or_endpoint=url, **kwargs)


class WeChatComponent(BaseWeChatComponent):
    PRE_AUTH_URL = "https://mp.weixin.qq.com/cgi-bin/componentloginpage"

    def get_pre_auth_url(self, redirect_uri):
        redirect_uri = quote(redirect_uri, safe=b"")
        return f"{self.PRE_AUTH_URL}?component_appid={self.component_appid}&pre_auth_code={self.create_preauthcode()['pre_auth_code']}&redirect_uri={redirect_uri}"

    def get_pre_auth_url_m(self, redirect_uri):
        """
        快速获取pre auth url，可以直接微信中发送该链接，直接授权
        """
        url = "https://mp.weixin.qq.com/safe/bindcomponent?action=bindcomponent&auth_type=3&no_scan=1&"
        redirect_uri = quote(redirect_uri, safe="")
        return f"{url}component_appid={self.component_appid}&pre_auth_code={self.create_preauthcode()['pre_auth_code']}&redirect_uri={redirect_uri}"

    def create_preauthcode(self):
        """
        获取预授权码
        """
        return self.post(
            "/component/api_create_preauthcode",
            data={"component_appid": self.component_appid},
        )

    def _query_auth(self, authorization_code):
        """
        使用授权码换取公众号的授权信息

        :params authorization_code: 授权code,会在授权成功时返回给第三方平台，详见第三方平台授权流程说明
        """
        return self.post(
            "/component/api_query_auth",
            data={
                "component_appid": self.component_appid,
                "authorization_code": authorization_code,
            },
        )

    def query_auth(self, authorization_code):
        """
        使用授权码换取公众号的授权信息,同时储存token信息

        :params authorization_code: 授权code,会在授权成功时返回给第三方平台，详见第三方平台授权流程说明
        """
        result = self._query_auth(authorization_code)

        assert (
            result is not None and "authorization_info" in result and "authorizer_appid" in result["authorization_info"]
        )

        authorizer_appid = result["authorization_info"]["authorizer_appid"]
        if (
            "authorizer_access_token" in result["authorization_info"]
            and result["authorization_info"]["authorizer_access_token"]
        ):
            access_token = result["authorization_info"]["authorizer_access_token"]
            access_token_key = f"{authorizer_appid}_access_token"
            expires_in = 7200
            if "expires_in" in result["authorization_info"]:
                expires_in = result["authorization_info"]["expires_in"]
            self.session.set(access_token_key, access_token, expires_in)
        if (
            "authorizer_refresh_token" in result["authorization_info"]
            and result["authorization_info"]["authorizer_refresh_token"]
        ):
            refresh_token = result["authorization_info"]["authorizer_refresh_token"]
            refresh_token_key = f"{authorizer_appid}_refresh_token"
            self.session.set(refresh_token_key, refresh_token)  # refresh_token 需要永久储存，不建议使用内存储存，否则每次重启服务需要重新扫码授权
        return result

    def refresh_authorizer_token(self, authorizer_appid, authorizer_refresh_token):
        """
        获取（刷新）授权公众号的令牌

        :params authorizer_appid: 授权方appid
        :params authorizer_refresh_token: 授权方的刷新令牌
        """
        return self.post(
            "/component/api_authorizer_token",
            data={
                "component_appid": self.component_appid,
                "authorizer_appid": authorizer_appid,
                "authorizer_refresh_token": authorizer_refresh_token,
            },
        )

    def get_authorizer_info(self, authorizer_appid):
        """
        获取授权方的账户信息

        :params authorizer_appid: 授权方appid
        """
        return self.post(
            "/component/api_get_authorizer_info",
            data={
                "component_appid": self.component_appid,
                "authorizer_appid": authorizer_appid,
            },
        )

    def get_authorizer_list(self, offset=0, count=500):
        """
        拉取所有已授权的帐号信息

        :params offset: 偏移位置/起始位置
        :params count: 拉取数量
        """
        return self.post(
            "/component/api_get_authorizer_list",
            data={
                "component_appid": self.component_appid,
                "offset": offset,
                "count": count,
            },
        )

    def get_authorizer_option(self, authorizer_appid, option_name):
        """
        获取授权方的选项设置信息

        :params authorizer_appid: 授权公众号appid
        :params option_name: 选项名称
        """
        return self.post(
            "/component/api_get_authorizer_option",
            data={
                "component_appid": self.component_appid,
                "authorizer_appid": authorizer_appid,
                "option_name": option_name,
            },
        )

    def set_authorizer_option(self, authorizer_appid, option_name, option_value):
        """
        设置授权方的选项信息

        :params authorizer_appid: 授权公众号appid
        :params option_name: 选项名称
        :params option_value: 设置的选项值
        """
        return self.post(
            "/component/api_set_authorizer_option",
            data={
                "component_appid": self.component_appid,
                "authorizer_appid": authorizer_appid,
                "option_name": option_name,
                "option_value": option_value,
            },
        )

    def get_client_by_appid(self, authorizer_appid):
        """
        通过 authorizer_appid 获取 Client 对象

        :params authorizer_appid: 授权公众号appid
        """
        access_token_key = f"{authorizer_appid}_access_token"
        refresh_token_key = f"{authorizer_appid}_refresh_token"
        access_token = self.session.get(access_token_key)
        refresh_token = self.session.get(refresh_token_key)
        assert refresh_token

        if not access_token:
            ret = self.refresh_authorizer_token(authorizer_appid, refresh_token)
            access_token = ret["authorizer_access_token"]
            access_token_key = f"{authorizer_appid}_access_token"
            expires_in = 7200
            if "expires_in" in ret:
                expires_in = ret["expires_in"]
            self.session.set(access_token_key, access_token, expires_in)

        return WeChatComponentClient(authorizer_appid, self, session=self.session)

    def parse_message(self, msg, msg_signature, timestamp, nonce):
        """
        处理 wechat server 推送消息

        :params msg: 加密内容
        :params msg_signature: 消息签名
        :params timestamp: 时间戳
        :params nonce: 随机数
        """
        content = self.crypto.decrypt_message(msg, msg_signature, timestamp, nonce)
        message = xmltodict.parse(to_text(content))["xml"]
        message_type = message["InfoType"].lower()
        message_class = COMPONENT_MESSAGE_TYPES.get(message_type, ComponentUnknownMessage)
        msg = message_class(message)
        if msg.type == "component_verify_ticket":
            self.session.set("{}_{}".format(self.component_appid, msg.type), msg.verify_ticket)
        elif msg.type in ("authorized", "updateauthorized"):
            msg.query_auth_result = self.query_auth(msg.authorization_code)
        return msg

    def get_component_oauth(self, authorizer_appid):
        """
        代公众号 OAuth 网页授权

        :params authorizer_appid: 授权公众号appid
        """
        return ComponentOAuth(self, authorizer_appid)


class ComponentOAuth:
    """微信开放平台 代公众号 OAuth 网页授权

    详情请参考
    https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&t=resource/res_list&verify=1&id=open1419318590
    """

    API_BASE_URL = "https://api.weixin.qq.com/"
    OAUTH_BASE_URL = "https://open.weixin.qq.com/connect/"

    def __init__(self, component, app_id):
        """
        :param component: WeChatComponent
        :param app_id: 微信公众号 app_id
        """
        self._http = requests.Session()
        self.app_id = app_id
        self.component = component

    def get_authorize_url(self, redirect_uri, scope="snsapi_base", state=""):
        """

        :param redirect_uri: 重定向地址，需要urlencode，这里填写的应是服务开发方的回调地址
        :param scope: 可选，微信公众号 OAuth2 scope，默认为 ``snsapi_base``
        :param state: 可选，重定向后会带上state参数，开发者可以填写任意参数值，最多128字节
        """
        redirect_uri = quote(redirect_uri, safe=b"")
        url_list = [
            self.OAUTH_BASE_URL,
            "oauth2/authorize?appid=",
            self.app_id,
            "&redirect_uri=",
            redirect_uri,
            "&response_type=code&scope=",
            scope,
        ]
        if state:
            url_list.extend(["&state=", state])
        url_list.extend(
            [
                "&component_appid=",
                self.component.component_appid,
            ]
        )
        url_list.append("#wechat_redirect")
        return "".join(url_list)

    def fetch_access_token(self, code):
        """获取 access_token

        :param code: 授权完成跳转回来后 URL 中的 code 参数
        :return: JSON 数据包
        """
        res = self._get(
            "sns/oauth2/component/access_token",
            params={
                "appid": self.app_id,
                "component_appid": self.component.component_appid,
                "component_access_token": self.component.access_token,
                "code": code,
                "grant_type": "authorization_code",
            },
        )
        self.access_token = res["access_token"]
        self.open_id = res["openid"]
        self.refresh_token = res["refresh_token"]
        self.expires_in = res["expires_in"]
        self.scope = res["scope"]
        return res

    def refresh_access_token(self, refresh_token):
        """刷新 access token

        :param refresh_token: OAuth2 refresh token
        :return: JSON 数据包
        """
        res = self._get(
            "sns/oauth2/component/refresh_token",
            params={
                "appid": self.app_id,
                "grant_type": "refresh_token",
                "refresh_token": refresh_token,
                "component_appid": self.component.component_appid,
                "component_access_token": self.component.access_token,
            },
        )
        self.access_token = res["access_token"]
        self.open_id = res["openid"]
        self.refresh_token = res["refresh_token"]
        self.expires_in = res["expires_in"]
        self.scope = res["scope"]
        return res

    def get_user_info(self, openid=None, access_token=None, lang="zh_CN"):
        """获取用户基本信息（需授权作用域为snsapi_userinfo）

        如果网页授权作用域为snsapi_userinfo，则此时开发者可以通过access_token和openid拉取用户信息了。

        :param openid: 可选，微信 openid，默认获取当前授权用户信息
        :param access_token: 可选，access_token，默认使用当前授权用户的 access_token
        :param lang: 可选，语言偏好, 默认为 ``zh_CN``
        :return: JSON 数据包
        """
        openid = openid or self.open_id
        access_token = access_token or self.access_token
        return self._get(
            "sns/userinfo",
            params={"access_token": access_token, "openid": openid, "lang": lang},
        )

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            url = f"{self.API_BASE_URL}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if isinstance(kwargs.get("data", ""), dict):
            body = json.dumps(kwargs["data"], ensure_ascii=False)
            body = body.encode("utf-8")
            kwargs["data"] = body

        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatOAuthException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )

        return self._handle_result(res, method=method, url=url, **kwargs)

    def _handle_result(self, res, method=None, url=None, **kwargs):
        result = json.loads(res.content.decode("utf-8", "ignore"), strict=False)
        if "errcode" in result:
            result["errcode"] = int(result["errcode"])

        if "errcode" in result and result["errcode"] != 0:
            errcode = result["errcode"]
            errmsg = result.get("errmsg", errcode)
            if self.component.auto_retry and errcode in (
                WeChatErrorCode.INVALID_CREDENTIAL.value,
                WeChatErrorCode.INVALID_ACCESS_TOKEN.value,
                WeChatErrorCode.EXPIRED_ACCESS_TOKEN.value,
            ):
                logger.info("Component access token expired, fetch a new one and retry request")
                self.component.fetch_access_token()
                kwargs["params"]["component_access_token"] = self.component.access_token
                return self._request(method=method, url_or_endpoint=url, **kwargs)
            elif errcode == WeChatErrorCode.OUT_OF_API_FREQ_LIMIT.value:
                # api freq out of limit
                raise APILimitedException(errcode, errmsg, client=self, request=res.request, response=res)
            else:
                raise WeChatComponentOAuthException(errcode, errmsg, client=self, request=res.request, response=res)
        return result

    def _get(self, url, **kwargs):
        return self._request(method="get", url_or_endpoint=url, **kwargs)



# -*- coding: utf-8 -*-


from wechatpy.utils import to_text
from wechatpy.client.api.base import BaseWeChatAPI


class WeChatGroup(BaseWeChatAPI):
    def create(self, name):
        """
        创建分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param name: 分组名字（30个字符以内）
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.create('New Group')

        """
        name = to_text(name)
        return self._post("groups/create", data={"group": {"name": name}})

    def get(self, user_id=None):
        """
        查询所有分组或查询用户所在分组 ID

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param user_id: 用户 ID，提供时查询该用户所在分组，否则查询所有分组
        :return: 所有分组列表或用户所在分组 ID

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            group = client.group.get('openid')

        """
        if user_id is None:
            res = self._get("groups/get", result_processor=lambda x: x["groups"])
        else:
            res = self._post(
                "groups/getid",
                data={"openid": user_id},
                result_processor=lambda x: x["groupid"],
            )
        return res

    def update(self, group_id, name):
        """
        修改分组名

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param group_id: 分组id，由微信分配
        :param name: 分组名字（30个字符以内）
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.update(1234, 'New Name')

        """
        name = to_text(name)
        return self._post("groups/update", data={"group": {"id": int(group_id), "name": name}})

    def move_user(self, user_id, group_id):
        """
        移动用户分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param user_id: 用户 ID, 可以是单个或者列表，为列表时为批量移动用户分组
        :param group_id: 分组 ID
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.move_user('openid', 1234)

        """
        data = {"to_groupid": group_id}
        if isinstance(user_id, (tuple, list)):
            endpoint = "groups/members/batchupdate"
            data["openid_list"] = user_id
        else:
            endpoint = "groups/members/update"
            data["openid"] = user_id
        return self._post(endpoint, data=data)

    def delete(self, group_id):
        """
        删除分组

        详情请参考
        http://mp.weixin.qq.com/wiki/0/56d992c605a97245eb7e617854b169fc.html

        :param group_id: 分组 ID
        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            res = client.group.delete(1234)

        """
        return self._post("groups/delete", data={"group": {"id": group_id}})



# -*- coding: utf-8 -*-


import json
import datetime

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatMarketing(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/marketing/"

    def add_user_action_sets(self, _type, name, description, version="v1.0"):
        """
        创建数据源
        https://wximg.qq.com/wxp/pdftool/get.html?id=rkalQXDBM&pa=39

        :param _type: 用户行为源类型
        :param name: 用户行为源名称 必填
        :param description: 用户行为源描述，字段长度最小 1 字节，长度最大 128 字节
        :param version: 版本号 v1.0
        :return: 数据源唯一ID
        """
        return self._post(
            "user_action_sets/add",
            params={"version": version},
            json=optionaldict(type=_type, name=name, description=description, version=version),
            result_processor=lambda x: x["data"]["user_action_set_id"],
        )

    def get_user_action_sets(self, user_action_set_id, version="v1.0"):
        """
        获取数据源信息

        :param user_action_set_id: 数据源唯一ID
        :param version: 版本号 v1.0
        """
        return self._get(
            "user_action_sets/get",
            params={"version": version, "user_action_set_id": user_action_set_id},
            result_processor=lambda x: x["data"]["list"],
        )

    def add_user_actions(self, actions=(), version="v1.0"):
        """
        回传数据

        https://wximg.qq.com/wxp/pdftool/get.html?id=rkalQXDBM&pa=39

        :param actions: 用户行为源类型
        :param version: 版本号 v1.0
        """
        return self._post("user_actions/add", params={"version": version}, json={"actions": actions})

    def get_user_action_set_reports(
        self,
        user_action_set_id,
        start_date=None,
        end_date=None,
        time_granularity=None,
        aggregation=None,
        version="v1.0",
    ):
        """
        数据源报表查询

        :param user_action_set_id: 数据源唯一ID
        :param start_date: 开始日期 默认今天
        :param end_date: 结束日期 默认今天
        :param time_granularity: 时间粒度 枚举 {'DAILY'（按天） 'HOURLY'（按小时）}
        :param aggregation: 聚合纬度 枚举 {'DOMAIN' 'ACTION_TYPE'}
        :param version: 版本号 v1.0
        :return:
        """
        today = datetime.date.today()
        if start_date is None:
            start_date = today
        if end_date is None:
            end_date = today
        if isinstance(start_date, datetime.date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, datetime.date):
            end_date = end_date.strftime("%Y-%m-%d")

        if time_granularity not in ("DAILY", "HOURLY"):
            time_granularity = None

        if aggregation not in ("DOMAIN", "ACTION_TYPE"):
            aggregation = None

        return self._get(
            "user_action_set_reports/get",
            params=optionaldict(
                user_action_set_id=user_action_set_id,
                date_range=json.dumps({"start_date": start_date, "end_date": end_date}),
                time_granularity=time_granularity,
                aggregation=aggregation,
                version=version,
            ),
            result_processor=lambda x: x["data"],
        )

    def get_ad_leads(
        self,
        start_date=None,
        end_date=None,
        filtering=(),
        page=1,
        page_size=100,
        version="v1.0",
    ):
        """
         获取朋友圈销售线索数据接口

        :param start_date: 开始日期 默认今天
        :param end_date: 结束日期 默认今天
        :param filtering: 过滤条件 [{field: 过滤字段, operator: 操作符, values: 字段取值}]
        :param page: 页码，获取指定页数据
        :param page_size: 一页获取的数据条数(1-100)
        :param version: 版本号 v1.0
        """
        today = datetime.date.today()
        if start_date is None:
            start_date = today
        if end_date is None:
            end_date = today
        if isinstance(start_date, datetime.date):
            start_date = start_date.strftime("%Y-%m-%d")
        if isinstance(end_date, datetime.date):
            end_date = end_date.strftime("%Y-%m-%d")

        return self._get(
            "wechat_ad_leads/get",
            params=optionaldict(
                date_range=json.dumps({"start_date": start_date, "end_date": end_date}),
                filtering=json.dumps(filtering) if filtering else None,
                page=page,
                page_size=page_size,
                version=version,
            ),
            result_processor=lambda x: x["data"],
        )



# -*- coding: utf-8 -*-

import datetime
from operator import itemgetter

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatDataCube(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/datacube/"

    @classmethod
    def _to_date_str(cls, date):
        if isinstance(date, datetime.date):
            return date.strftime("%Y-%m-%d")
        elif isinstance(date, str):
            return date
        else:
            raise ValueError("Can not convert %s type to str", type(date))

    def get_user_summary(self, begin_date, end_date):
        """
        获取用户增减数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/User_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getusersummary",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
        )
        return res["list"]

    def get_user_cumulate(self, begin_date, end_date):
        """
        获取累计用户数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/User_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getusercumulate",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_interface_summary(self, begin_date, end_date):
        """
        获取接口分析数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Analytics_API.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getinterfacesummary",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_interface_summary_hour(self, begin_date, end_date):
        """
        获取接口分析分时数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Analytics_API.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getinterfacesummaryhour",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_article_summary(self, begin_date, end_date):
        """
        获取图文群发每日数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getarticlesummary",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_article_total(self, begin_date, end_date):
        """
        获取图文群发总数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getarticletotal",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_user_read(self, begin_date, end_date):
        """
        获取图文统计数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getuserread",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_user_read_hour(self, begin_date, end_date):
        """
        获取图文分时统计数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getuserreadhour",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_user_share(self, begin_date, end_date):
        """
        获取图文分享转发数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getusershare",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_user_share_hour(self, begin_date, end_date):
        """
        获取图文分享转发分时数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Graphic_Analysis_Data_Interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getusersharehour",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg(self, begin_date, end_date):
        """
        获取消息发送概况数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsg",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_hour(self, begin_date, end_date):
        """
        获取消息发送分时数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsghour",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_week(self, begin_date, end_date):
        """
        获取消息发送周数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsgweek",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_month(self, begin_date, end_date):
        """
        获取消息发送月数据
        详情请参考
        http://mp.weixin.qq.com/wiki/12/32d42ad542f2e4fc8a8aa60e1bce9838.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsgmonth",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_dist(self, begin_date, end_date):
        """
        获取消息发送分布数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsgdist",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_dist_week(self, begin_date, end_date):
        """
        获取消息发送分布周数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsgdistweek",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res

    def get_upstream_msg_dist_month(self, begin_date, end_date):
        """
        获取消息发送分布月数据
        详情请参考
        https://developers.weixin.qq.com/doc/offiaccount/Analytics/Message_analysis_data_interface.html

        :param begin_date: 起始日期
        :param end_date: 结束日期
        :return: 统计数据列表
        """
        res = self._post(
            "getupstreammsgdistmonth",
            data={
                "begin_date": self._to_date_str(begin_date),
                "end_date": self._to_date_str(end_date),
            },
            result_processor=itemgetter("list"),
        )
        return res



# -*- coding: utf-8 -*-
"""
    wechatpy.oauth
    ~~~~~~~~~~~~~~~

    This module provides OAuth2 library for WeChat

    :copyright: (c) 2014 by messense.
    :license: MIT, see LICENSE for more details.
"""
import json
from urllib.parse import quote

import requests

from wechatpy.exceptions import WeChatOAuthException


class WeChatOAuth:
    """微信公众平台 OAuth 网页授权

    详情请参考
    https://open.weixin.qq.com/cgi-bin/showdocument?action=dir_list&t=resource/res_list&verify=1&id=open1419316505
    """

    API_BASE_URL = "https://api.weixin.qq.com/"
    OAUTH_BASE_URL = "https://open.weixin.qq.com/connect/"

    def __init__(self, app_id, secret, redirect_uri, scope="snsapi_base", state=""):
        """

        :param app_id: 微信公众号 app_id
        :param secret: 微信公众号 secret
        :param redirect_uri: OAuth2 redirect URI
        :param scope: 可选，微信公众号 OAuth2 scope，默认为 ``snsapi_base``
        :param state: 可选，微信公众号 OAuth2 state
        """
        self.app_id = app_id
        self.secret = secret
        self.redirect_uri = redirect_uri
        self.scope = scope
        self.state = state
        self._http = requests.Session()

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(("http://", "https://")):
            url = f"{self.API_BASE_URL}{url_or_endpoint}"
        else:
            url = url_or_endpoint

        if isinstance(kwargs.get("data", ""), dict):
            body = json.dumps(kwargs["data"], ensure_ascii=False)
            body = body.encode("utf-8")
            kwargs["data"] = body

        res = self._http.request(method=method, url=url, **kwargs)
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            raise WeChatOAuthException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response,
            )
        result = json.loads(res.content.decode("utf-8", "ignore"), strict=False)

        if "errcode" in result and result["errcode"] != 0:
            errcode = result["errcode"]
            errmsg = result["errmsg"]
            raise WeChatOAuthException(errcode, errmsg, client=self, request=res.request, response=res)

        return result

    def _get(self, url, **kwargs):
        return self._request(method="get", url_or_endpoint=url, **kwargs)

    @property
    def authorize_url(self):
        """获取授权跳转地址

        :return: URL 地址
        """
        redirect_uri = quote(self.redirect_uri, safe=b"")
        url_list = [
            self.OAUTH_BASE_URL,
            "oauth2/authorize?appid=",
            self.app_id,
            "&redirect_uri=",
            redirect_uri,
            "&response_type=code&scope=",
            self.scope,
        ]
        if self.state:
            url_list.extend(["&state=", self.state])
        url_list.append("#wechat_redirect")
        return "".join(url_list)

    @property
    def qrconnect_url(self):
        """生成扫码登录地址

        :return: URL 地址
        """
        redirect_uri = quote(self.redirect_uri, safe=b"")
        url_list = [
            self.OAUTH_BASE_URL,
            "qrconnect?appid=",
            self.app_id,
            "&redirect_uri=",
            redirect_uri,
            "&response_type=code&scope=",
            "snsapi_login",  # scope
        ]
        if self.state:
            url_list.extend(["&state=", self.state])
        url_list.append("#wechat_redirect")
        return "".join(url_list)

    def fetch_access_token(self, code):
        """获取 access_token

        :param code: 授权完成跳转回来后 URL 中的 code 参数
        :return: JSON 数据包
        """
        res = self._get(
            "sns/oauth2/access_token",
            params={
                "appid": self.app_id,
                "secret": self.secret,
                "code": code,
                "grant_type": "authorization_code",
            },
        )
        self.access_token = res["access_token"]
        self.open_id = res["openid"]
        self.refresh_token = res["refresh_token"]
        self.expires_in = res["expires_in"]
        return res

    def refresh_access_token(self, refresh_token):
        """刷新 access token

        :param refresh_token: OAuth2 refresh token
        :return: JSON 数据包
        """
        res = self._get(
            "sns/oauth2/refresh_token",
            params={
                "appid": self.app_id,
                "grant_type": "refresh_token",
                "refresh_token": refresh_token,
            },
        )
        self.access_token = res["access_token"]
        self.open_id = res["openid"]
        self.refresh_token = res["refresh_token"]
        self.expires_in = res["expires_in"]
        return res

    def get_user_info(self, openid=None, access_token=None, lang="zh_CN"):
        """获取用户信息

        :param openid: 可选，微信 openid，默认获取当前授权用户信息
        :param access_token: 可选，access_token，默认使用当前授权用户的 access_token
        :param lang: 可选，语言偏好, 默认为 ``zh_CN``
        :return: JSON 数据包
        """
        openid = openid or self.open_id
        access_token = access_token or self.access_token
        return self._get(
            "sns/userinfo",
            params={"access_token": access_token, "openid": openid, "lang": lang},
        )

    def check_access_token(self, openid=None, access_token=None):
        """检查 access_token 有效性

        :param openid: 可选，微信 openid，默认获取当前授权用户信息
        :param access_token: 可选，access_token，默认使用当前授权用户的 access_token
        :return: 有效返回 True，否则 False
        """
        openid = openid or self.open_id
        access_token = access_token or self.access_token
        res = self._get("sns/auth", params={"access_token": access_token, "openid": openid})
        if res["errcode"] == 0:
            return True
        return False



# -*- coding: utf-8 -*-

from urllib.parse import quote

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatOAuth(BaseWeChatAPI):
    OAUTH_BASE_URL = "https://open.weixin.qq.com/connect/oauth2/authorize"

    def authorize_url(self, redirect_uri, state=None):
        """
        构造网页授权链接
        详情请参考
        https://work.weixin.qq.com/api/doc#90000/90135/91022

        :param redirect_uri: 授权后重定向的回调链接地址
        :param state: 重定向后会带上 state 参数
        :return: 返回的 JSON 数据包
        """
        redirect_uri = quote(redirect_uri, safe=b"")
        url_list = [
            self.OAUTH_BASE_URL,
            "?appid=",
            self._client.corp_id,
            "&redirect_uri=",
            redirect_uri,
            "&response_type=code&scope=snsapi_base",
        ]
        if state:
            url_list.extend(["&state=", state])
        url_list.append("#wechat_redirect")
        return "".join(url_list)

    def get_user_info(self, code):
        """
        获取访问用户身份
        详情请参考
        https://work.weixin.qq.com/api/doc#90000/90135/91023

        :param code: 通过成员授权获取到的code
        :return: 返回的 JSON 数据包
        """

        return self._get(
            "user/getuserinfo",
            params={
                "code": code,
            },
        )



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantCommon(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def upload_image(self, filename, image_data):
        res = self._post(
            "merchant/common/upload_img",
            params={"filename": filename},
            data=image_data,
            result_processor=lambda x: x["image_url"],
        )
        return res



# -*- coding: utf-8 -*-

from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatScan(BaseWeChatAPI):
    API_BASE_URL = "https://api.weixin.qq.com/scan/"

    def get_merchant_info(self):
        """
        获取商户信息

        详情请参考
        http://mp.weixin.qq.com/wiki/6/c61604ff6890d386d6227945ad4a68d2.html

        :return: 返回的 JSON 数据包

        使用示例::

            from wechatpy import WeChatClient

            client = WeChatClient('appid', 'secret')
            info = client.scan.get_merchant_info()
        """
        return self._get("merchantinfo/get")

    def create_product(self, product_data):
        """
        创建商品

        详情请参考
        http://mp.weixin.qq.com/wiki/6/c61604ff6890d386d6227945ad4a68d2.html

        :return: 返回的 JSON 数据包
        """
        return self._post("product/create", data=product_data)

    def modify_product_status(self, standard, key, status):
        """
        提交审核/取消发布商品

        详情请参考
        http://mp.weixin.qq.com/wiki/15/1007691d0f1c10a0588c6517f12ed70f.html

        :param standard: 商品编码标准
        :param key: 商品编码内容
        :param status: 设置发布状态。on 为提交审核，off 为取消发布
        :return: 返回的 JSON 数据包
        """
        data = {
            "keystandard": standard,
            "keystr": key,
            "status": status,
        }
        return self._post("product/modstatus", data=data)

    def publish_product(self, standard, key):
        """
        提交审核商品 shortcut 接口

        等同于调用 ``modify_product_status(standard, key, 'on')``
        """
        return self.modify_product_status(standard, key, "on")

    def unpublish_product(self, standard, key):
        """
        取消发布商品 shortcut 接口

        等同于调用 ``modify_product_status(standard, key, 'off')``
        """
        return self.modify_product_status(standard, key, "off")

    def set_test_whitelist(self, userids=None, usernames=None):
        """
        设置测试人员白名单

        注意：每次设置均被视为一次重置，而非增量设置。openid、微信号合计最多设置10个。

        详情请参考
        http://mp.weixin.qq.com/wiki/15/1007691d0f1c10a0588c6517f12ed70f.html

        :param userids: 可选，测试人员的 openid 列表
        :param usernames: 可选，测试人员的微信号列表
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(openid=userids, username=usernames)
        return self._post("testwhitelist/set", data=data)

    def get_product(self, standard, key):
        """
        查询商品信息

        详情请参考
        http://mp.weixin.qq.com/wiki/15/7fa787701295b884410b5163e13313af.html

        :param standard: 商品编码标准
        :param key: 商品编码内容
        :return: 返回的 JSON 数据包
        """
        data = {
            "keystandard": standard,
            "keystr": key,
        }
        return self._post("product/get", data=data)

    def list_product(self, offset=0, limit=10, status=None, key=None):
        """
        批量查询商品信息

        详情请参考
        http://mp.weixin.qq.com/wiki/15/7fa787701295b884410b5163e13313af.html

        :param offset: 可选，批量查询的起始位置，从 0 开始，包含该起始位置
        :param limit: 可选，批量查询的数量，默认为 10
        :param status: 可选，支持按状态拉取。on为发布状态，off为未发布状态，
                       check为审核中状态，reject为审核未通过状态，all为所有状态
        :param key: 支持按部分编码内容拉取。填写该参数后，可将编码内容中包含所传参数的商品信息拉出
        :return: 返回的 JSON 数据包
        """
        data = optionaldict(
            offset=offset,
            limit=limit,
            status=status,
            keystr=key,
        )
        return self._post("product/getlist", data=data)

    def update_product(self, product_data):
        """
        更新商品信息

        详情请参考
        http://mp.weixin.qq.com/wiki/15/7fa787701295b884410b5163e13313af.html

        :return: 返回的 JSON 数据包
        """
        return self._post("product/update", data=product_data)

    def clear_product(self, standard, key):
        """
        清除商品信息

        详情请参考
        http://mp.weixin.qq.com/wiki/15/7fa787701295b884410b5163e13313af.html

        :param standard: 商品编码标准
        :param key: 商品编码内容
        :return: 返回的 JSON 数据包
        """
        data = {
            "keystandard": standard,
            "keystr": key,
        }
        return self._post("product/clear", data=data)

    def check_ticket(self, ticket):
        """
        检查 wxticket 参数有效性

        详情请参考
        http://mp.weixin.qq.com/wiki/15/7fa787701295b884410b5163e13313af.html

        :param ticket: 请求 URL 中带上的 wxticket 参数
        :return: 返回的 JSON 数据包
        """
        return self._post("scanticket/check", data={"ticket": ticket})



# -*- coding: utf-8 -*-

from wechatpy.client.api.base import BaseWeChatAPI


class MerchantExpress(BaseWeChatAPI):

    API_BASE_URL = "https://api.weixin.qq.com/"

    def add(self, delivery_template):
        return self._post("merchant/express/add", data={"delivery_template": delivery_template})

    def delete(self, template_id):
        return self._post("merchant/express/del", data={"template_id": template_id})

    def update(self, template_id, delivery_template):
        return self._post(
            "merchant/express/update",
            data={"template_id": template_id, "delivery_template": delivery_template},
        )

    def get(self, template_id):
        res = self._post(
            "merchant/express/getbyid",
            data={"template_id": template_id},
            result_processor=lambda x: x["template_info"],
        )
        return res

    def get_all(self):
        res = self._get("merchant/express/getall", result_processor=lambda x: x["template_info"])
        return res



# -*- coding: utf-8 -*-


from optionaldict import optionaldict

from wechatpy.client.api.base import BaseWeChatAPI


class WeChatKFMessage(BaseWeChatAPI):
    """
    发送微信客服消息

    https://work.weixin.qq.com/api/doc/90000/90135/94677

    支持：
    * 文本消息
    * 图片消息
    * 语音消息
    * 视频消息
    * 文件消息
    * 图文链接
    * 小程序
    * 菜单消息
    * 地理位置
    """

    def send(self, user_id, open_kfid, msgid="", msg=None):
        """
        当微信客户处于“新接入待处理”或“由智能助手接待”状态下，可调用该接口给用户发送消息。
        注意仅当微信客户在主动发送消息给客服后的48小时内，企业可发送消息给客户，最多可发送5条消息；若用户继续发送消息，企业可再次下发消息。
        支持发送消息类型：文本、图片、语音、视频、文件、图文、小程序、菜单消息、地理位置。

        :param user_id: 指定接收消息的客户UserID
        :param open_kfid: 指定发送消息的客服帐号ID
        :param msgid: 指定消息ID
        :param tag_ids: 标签ID列表。
        :param msg: 发送消息的 dict 对象
        :type msg: dict | None
        :return: 接口调用结果
        """
        msg = msg or {}
        data = {
            "touser": user_id,
            "open_kfid": open_kfid,
        }
        if msgid:
            data["msgid"] = msgid
        data.update(msg)
        return self._post("kf/send_msg", data=data)

    def send_text(self, user_id, open_kfid, content, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "text", "text": {"content": content}},
        )

    def send_image(self, user_id, open_kfid, media_id, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "image", "image": {"media_id": media_id}},
        )

    def send_voice(self, user_id, open_kfid, media_id, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "voice", "voice": {"media_id": media_id}},
        )

    def send_video(self, user_id, open_kfid, media_id, msgid=""):
        video_data = optionaldict()
        video_data["media_id"] = media_id

        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "video", "video": dict(video_data)},
        )

    def send_file(self, user_id, open_kfid, media_id, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "file", "file": {"media_id": media_id}},
        )

    def send_articles_link(self, user_id, open_kfid, article, msgid=""):
        articles_data = {
            "title": article["title"],
            "desc": article["desc"],
            "url": article["url"],
            "thumb_media_id": article["thumb_media_id"],
        }
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={"msgtype": "news", "link": {"link": articles_data}},
        )

    def send_msgmenu(self, user_id, open_kfid, head_content, menu_list, tail_content, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={
                "msgtype": "msgmenu",
                "msgmenu": {"head_content": head_content, "list": menu_list, "tail_content": tail_content},
            },
        )

    def send_location(self, user_id, open_kfid, name, address, latitude, longitude, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={
                "msgtype": "location",
                "msgmenu": {"name": name, "address": address, "latitude": latitude, "longitude": longitude},
            },
        )

    def send_miniprogram(self, user_id, open_kfid, appid, title, thumb_media_id, pagepath, msgid=""):
        return self.send(
            user_id,
            open_kfid,
            msgid,
            msg={
                "msgtype": "miniprogram",
                "msgmenu": {"appid": appid, "title": title, "thumb_media_id": thumb_media_id, "pagepath": pagepath},
            },
        )
